@@ -830,17 +830,31 @@ Bool make_current_gl_handler(Gl_handler gl_handler)
 #else
     glXWaitGL();
     glXWaitX();
-    glGetError(); /* remove existing errors */
+    //glGetError(); /* remove existing errors */
     /* wb104 18 Jul 2013 */
     /* on Mac at least if above is not used then last "on" spectrum is not drawn */
+    /* that is because glGetError() call below was throwing a 1282 (GL_INVALID_OPERATION) error */
+    /* but above glGetError() was crashing some Linux at least */
     ok = glXMakeCurrent(gl_handler_p->display, Tk_WindowId(gl_handler_p->tk_win),
 						gl_handler_p->context);
 #endif
 /*
     glDrawBuffer(GL_BACK);
 */
-    if (!ok || !(glGetError() == GL_NO_ERROR))
+    //if (!ok || !(glGetError() == GL_NO_ERROR))
+    //  return CCPN_FALSE;
+    if (!ok)
+    {
+      printf("not ok\n");
       return CCPN_FALSE;
+    }
+/*  wb104 17 Mar 2014 */
+/*  below often seems to be triggered on Mac (see above) but leaving it out seems to cause no harm */
+/*
+    GLenum err = glGetError();
+    if (err != GL_NO_ERROR)
+      return CCPN_FALSE;
+*/
 
     return CCPN_TRUE;
 }
