@@ -656,7 +656,10 @@ def findDataStoringFromFilepath(project, fullFilePath, preferDataUrls=None,
       dataUrl = ll[-1][1]
     
     # get filePath
-    ss = uniIo.joinPath(dataUrl.url.path, '') # removes file separator from end
+    if ss == '/':
+      ss = ''
+    else:
+      ss = uniIo.joinPath(dataUrl.url.path, '') # removes file separator from end, but not if just '/'
     filePath = fullFilePath[len(ss)+1:]
   
   else:
