@@ -278,7 +278,7 @@ class ViewStructureFrame(ScrolledCanvas):
     x = event.x
     y = event.y
 
-    if self.initialX is None:
+    if not self.initialX:
       self.initialX = x
       self.initialY = y
 
@@ -328,7 +328,7 @@ class ViewStructureFrame(ScrolledCanvas):
   def mouseButton2(self, event):
     
     self.stop()
-    if self.initialX is None:
+    if not self.initialX:
       self.initialX = event.x
       self.initialY = event.y
 
@@ -536,12 +536,14 @@ class ViewStructureFrame(ScrolledCanvas):
       cAtomDict = self.cAtomDict
       model = self.model
       structure = model.structureEnsemble
-      for chain in structure.coordChains:
-        for residue in chain.residues:
-           for atom in residue.atoms:
-             coord = atom.findFirstCoord(model=model)
-             cAtom = cAtomDict[atom]
-             cAtom.setCoords((coord.x, coord.y, coord.z))
+      
+      coordinates = model.coordinates
+      offset = 0
+      for atom in structure.orderedAtoms:
+        next = offset + 3
+        cAtom = cAtomDict[atom]
+        cAtom.setCoords(coordinates[offset:next])
+        offset = next
     
       aX = self.xRotation
       aY = self.yRotation
@@ -564,6 +566,8 @@ class ViewStructureFrame(ScrolledCanvas):
       newBond = StructBond.StructBond
       
       model = self.model
+      ensembleAtoms = model.structureEnsemble.orderedAtoms
+      modelCoordinates = model.coordinates
       if model:
  
         cAtomDict = {}
@@ -600,15 +604,14 @@ class ViewStructureFrame(ScrolledCanvas):
               chemAtomDict[atom1.chemAtom] = atom1
  
             for atom1 in residue.atoms:
-              coord  = atom1.findFirstCoord(model=model)
-              
-              if not coord:
-                continue 
-              
               symbol = atom1.elementSymbol
               bonds1 = atom1.chemAtom.chemBonds
               color, label, size  = getAtomDisplayScheme(atom1)
-              cAtom  = newAtom(size, '', label, [coord.x, coord.y, coord.z], color)
+              # The next could be speeded up by looping over ensembleAtoms,
+              # but why trouble to rewrite the entire loop
+              offset = 3*ensembleAtoms.index(atom1)
+              tt = modelCoordinates[offset:offset+3]
+              cAtom  = newAtom(size, '', label, tt, color)
                             
               atomColors[cAtom] = color
               cAtomDict[atom1] = cAtom
