@@ -564,7 +564,16 @@ class Analysis:
         data.append((axisRegion, tiedAxisPanel.findFirstAxisRegion()))
 
     for axisRegion, tiedAxisRegion in data:
-      tiedAxisRegion.region = axisRegion.region
+      if axisRegion.axisPanel.label in ('x', 'y') and tiedAxisRegion.axisPanel.label in ('x', 'y'):
+        # change entire region
+        tiedAxisRegion.region = axisRegion.region
+      else:
+        # change only center point, not extent
+        (r1, r2) = tiedAxisRegion.region
+        dr = 0.5 * (r2 - r1)
+        (r1, r2) = axisRegion.region
+        c = 0.5 * (r1 + r2)
+        tiedAxisRegion.region = (c - dr, c + dr)
 
     # if you don't do after_idle then it gets into an infinite loop
     self.after_idle(lambda: self.registerNotify(self.updateTiedAxesAfter, 'ccpnmr.Analysis.AxisRegion', 'setRegion'))
@@ -1047,54 +1056,8 @@ class Analysis:
       chainMapping.delete()
 
   def initAtomSetMappings(self):
-
-    atomSetDict = {}
-    for atomSet in self.nmrProject.atomSets:
-      atomSetDict[atomSet.serial] = atomSet
-
-    msDict = {}
-    for molSystem in self.project.molSystems:
-      msDict[molSystem.code] = {}
-      
-      for chain in molSystem.chains:
-        msDict[molSystem.code][chain.code] = {}
-        
-        for residue in chain.residues:
-          msDict[molSystem.code][chain.code][residue.seqId] = 1
     
-    updateMapping = AssignmentBasic.updateAtomSetMapping
-
-    for chainMapping in self.analysisProject.chainMappings:
-      msc = chainMapping.molSystemCode
-      chc = chainMapping.chainCode
-      
-      if msDict.get(msc) is None:
-        chainMapping.delete()
-        continue
-      
-      if msDict[msc].get(chc) is None:
-        chainMapping.delete()
-        continue
-
-      for residueMapping in chainMapping.residueMappings:
-        if msDict[msc][chc].get(residueMapping.seqId) is None:
-          residueMapping.delete()
-          continue
-
-        for atomSetMapping in residueMapping.atomSetMappings:
-          if atomSetMapping.atomSetSerials:
-            atomSets = []
-
-            for serial in atomSetMapping.atomSetSerials:
-              atomSet = atomSetDict.get(serial)
-
-              if atomSet:
-                atomSets.append(atomSet)
-
-            if not atomSets:
-              continue
-
-            updateMapping(atomSetMapping, atomSets)
+    AssignmentBasic.initAtomSetMappings(self.analysisProject)
 
 
   def initShiftLists(self):
@@ -2257,9 +2220,10 @@ class Analysis:
 
   def changedPeakListSymbolStyle(self, analysisPeakList):
 
-    if analysisPeakList.symbolStyle == '+':
-      symbolCode = 1
-    else:
+    symbolList = ['x', '+', 'o', '*']
+    try:
+      symbolCode = symbolList.index(analysisPeakList.symbolStyle)
+    except:
       symbolCode = 0
       
     analysisPeakList.peakList.cPeakList.setSymbol(symbolCode)
