@@ -695,3 +695,59 @@ def getBlankPeakList(spectrum):
     
     
   return peakList  
+
+def possiblePeakAssigments(peak, shiftRanges=None, tolerances=None,
+                           aliasing=True, findAssigned=False):
+  """ Get list of resonance that make possible assignments for a peak
+  NB could be made more sophisticated
+  NB does  not (yet) deal with TOCSY or intraresidue
+  """
+  
+  dataSource = peak.peakList.dataSource
+  numDim = dataSource.numDim
+  if shiftRanges is None:
+    shiftRanges = [None] * numDim
+  if tolerances is None:
+    tolerances = estimateAssignmentTolerances(dataSource)
+    tolerances = [None] * numDim
+  
+  
+  matchResDict = {}
+  for ii,peakDim in enumerate(peak.sortedPeakDims()):
+    ll = findMatchingPeakDimShifts(peakDim, shiftRanges[ii], 
+                                                   tolerances[ii], aliasing, 
+                                                   findAssigned)
+    matchResDict[peakDim.dim] = set(x.resonance for x in ll)
+  
+  
+  boundDimNos = []
+  for dim1,dim2 in ExperimentBasic.getOnebondDataDims(dataSource):
+   boundDimNos.append((dim1.dim, dim2.dim))
+  
+  
+  # NBNB Multiple runs to guard against one dimension being bound to two others.
+  changing = True
+  while changing:
+    changing = False
+    for dimNo,resonances in matchResDict.items():
+      for dimNo1, dimNo2 in boundDimNos:
+        if dimNo == dimNo1:
+          set1 = set()
+          set2 = set()
+          for res1 in matchResDict[dimNo1]:
+            for res2 in matchResDict[dimNo2]:
+              if areResonancesBound(res1, res2):
+                set1.add(res1)
+                set2.add(res2)
+          if (len(set1) < len(matchResDict[dimNo1]) or
+              len(set2) < len(matchResDict[dimNo2])):
+            changing = True
+            matchResDict[dimNo1] = set1
+            matchResDict[dimNo2] = set2
+    
+  #
+  return [val for key,val in sorted(matchResDict.items())]
+    
+  
+  
+
