@@ -65,7 +65,6 @@ except ImportError:
   from memops.universal.MessageReporter import showWarning, showYesNo
   #from memops.universal.DataEntry       import askString
 
-
 def updateDistConstraintFromPeakAssign(constraint):
   """
   Update the items/contributions of a distance constraint given
@@ -712,6 +711,9 @@ def copyFixedResonance(fixedResonance, nmrConstraintStore):
   NmrConstraint.FixedResonance
   """
   
+  tags = ('resonanceSerial', 'name', 'details', 'isotopeCode')
+  resonanceMap = {}
+  
   cSet = nmrConstraintStore
   
   if fixedResonance.resonanceSerial:
@@ -720,11 +722,7 @@ def copyFixedResonance(fixedResonance, nmrConstraintStore):
     if fixedResonance2:
       return fixedResonance2
     
-  serial       = fixedResonance.resonanceSerial
-  name         = fixedResonance.name
-  details      = fixedResonance.details
-  isotope      = fixedResonance.isotopeCode
-  resonanceSet = fixedResonance.resonanceSet
+    resonanceSet = fixedResonance.resonanceSet
 
   if resonanceSet:
     atomSets2 = [getFixedAtomSet(cSet, ass.atoms) for ass in resonanceSet.atomSets]
@@ -735,27 +733,34 @@ def copyFixedResonance(fixedResonance, nmrConstraintStore):
       fixedResonance2 = list(fixedResonance2.resonances)[index]
     
     else:
-      fixedResonance2 =  cSet.newFixedResonance(name=name,
-                                                 details=details,
-                                                 isotopeCode=isotope,
-                                                 resonanceSerial=serial)
-      fixedResonances2 = [fixedResonance2]
+      kw = {}
+      for tag in tags:
+        kw[tag] = getattr(fixedResonance,tag)
+      fixedResonance2 =  cSet.newFixedResonance(**kw)
+      #fixedResonances2 = [fixedResonance2]
+      resonanceMap[fixedResonance] = fixedResonance2
 
       for fixedResonance1 in resonanceSet.resonances:
         if fixedResonance1 is not fixedResonance:
-          fixedResonance3 = cSet.newFixedResonance(name=fixedResonance1.name,
-                                                                 details=fixedResonance1.details,
-                                                                 isotopeCode=fixedResonance1.isotopeCode,
-                                                                 resonanceSerial=fixedResonance1.resonanceSerial)
-          fixedResonances2.append(fixedResonance3)
+          kw = {}
+          for tag in tags:
+            kw[tag] = getattr(fixedResonance1,tag)
+          fixedResonance3 = cSet.newFixedResonance(**kw)
+          #fixedResonances2.append(fixedResonance3)
+          resonanceMap[fixedResonance1] = fixedResonance3
 
-      resonanceSet2 = cSet.newFixedResonanceSet(atomSets=atomSets2,resonances=fixedResonances2)
+      resonanceSet2 = cSet.newFixedResonanceSet(atomSets=atomSets2,
+                                                resonances=resonanceMap.values)
 
   else:
-    fixedResonance2 = cSet.newFixedResonance(name=name,
-                                             details=details,
-                                             isotopeCode=isotope,
-                                             resonanceSerial=serial)
+    kw = {}
+    for tag in tags:
+      kw[tag] = getattr(fixedResonance,tag)
+    fixedResonance2 = cSet.newFixedResonance(**kw)
+    resonanceMap[fixedResonance] = fixedResonance2
+  
+  for fr1, fr2 in resonanceMap.items():
+    transferCovalentlyBound(fr1, fr2)
   
   return fixedResonance2
 
@@ -1842,6 +1847,7 @@ def makeFixedResonance(nmrConstraintStore,resonance):
   Make a new fixed resonance for an NMR constraint top object based on a normal
   resonance. The fixed resonance will preserve assignment information at the
   time of constraint generation.
+  NB should work equally well for fixedResonance input
 
   .. describe:: Input
   
@@ -1855,7 +1861,7 @@ def makeFixedResonance(nmrConstraintStore,resonance):
   fixedResonance = nmrConstraintStore.newFixedResonance(resonanceSerial=resonance.serial,
                                                         isotopeCode=resonance.isotopeCode,
                                                         name=resonance.name)
-  
+  transferCovalentlyBound(resonance, fixedResonance)
   return fixedResonance
 
 def getFixedResonance(nmrConstraintStore,resonance):
@@ -1863,6 +1869,7 @@ def getFixedResonance(nmrConstraintStore,resonance):
   Find or create a fixed resonance for an NMR constraint top  object equivalent
   to the input normal resonance. The fixed resonance will preserve assignment
   information at the time of constraint generation.
+  NB should work equally well for fixedResonance input
   
   .. describe:: Input
   
@@ -2009,8 +2016,7 @@ def isResidueInRange(residue, residueRanges, dataDim):
 
 def getMeanPeakIntensity(peaks, intensityType='volume'):
   """
-  Calculate the mean of the unsigned intensities of input peaks. Also sets
-  peak.intensityValue on-the-fly attribute for quick access by Analysis.
+  Calculate the mean of the unsigned intensities of input peaks.
   
   .. describe:: Input
   
@@ -2029,13 +2035,9 @@ def getMeanPeakIntensity(peaks, intensityType='volume'):
     
     if intensity:
       value = abs(intensity.value)
-      peak.intensityValue = value
       sumV += value
       n    += 1.0
-      
-    else:
-      peak.intensityValue = 0.0
-      
+            
   return sumV/max(n,1.0)
 
 
@@ -2061,10 +2063,193 @@ def getNoeDistance(intensity, params):
 
   return dist, minDist, maxDist
 
+def getDistMinMax(intensityValue, intensityScale, resonances0, resonances1, distanceFunction, normalise=True, labelling=None):
+
+  if normalise:
+    isoCorr0 = 0.0
+    isoCorr1 = 0.0
+
+    weight0 = 1.0/len(resonances0) 
+    weight1 = 1.0/len(resonances1) 
+
+    for resonance, indirect in resonances0:
+      resonanceSet = resonance.resonanceSet
+
+      propAtoms = 1.0 
+
+      if indirect:
+        propAtoms = 0.0
+        for resonanceB in indirect:
+          if labelling:
+            frac = max(0.1, getResonancePairLabellingFraction(resonance, resonanceB, labelling) or 1.0)
+          else:
+            frac = 1.0  
+
+          propAtoms += frac
+
+      elif labelling:
+        propAtoms *= max(0.1, getResonanceLabellingFraction(resonance, labelling) or 1.0)
+
+      fac = weight0/propAtoms
+      isoCorr0 +=  fac
+
+    for resonance, indirect in resonances1:
+      resonanceSet = resonance.resonanceSet
+
+      propAtoms = 1.0 
+
+      if indirect:
+        propAtoms = 0.0
+        for resonanceB in indirect:
+          if labelling:
+            frac = max(0.1, getResonancePairLabellingFraction(resonance, resonanceB, labelling) or 1.0)
+          else:
+            frac = 1.0  
+
+          propAtoms += frac
+
+      elif labelling:
+        propAtoms *= max(0.1, getResonanceLabellingFraction(resonance, labelling) or 1.0)
+
+      fac = weight1/propAtoms
+      isoCorr1 += fac
+
+  else:
+    isoCorr0 = 1
+    isoCorr1 = 1
+
+  (dist,minDist,maxDist) = distanceFunction(intensityValue*isoCorr0*isoCorr1/intensityScale)
+
+  return (dist,minDist,maxDist)
+
+def findPeakConstraintPairs(peak, residueRanges=None, distDataDims=None, distIndices=None, indirectDims=None):
+
+  spectrum = peak.peakList.dataSource
+  experiment = spectrum.experiment
+  nmrProject = experiment.nmrProject
+
+  if distDataDims is None:
+    distDataDims = getThroughSpaceDataDims(spectrum)
+
+  if distIndices is None:
+    distIndices  = [dd.dim-1 for dd in distDataDims]
+
+  if indirectDims is None:
+    indirectDims = {}
+    for dataDims in getIndirectDataDims(spectrum):
+      if  set(dataDims) == set(distDataDims):
+        isotopesDict = getIndirectThroughSpaceIsotopes(experiment)
+
+        for dataDim in dataDims:
+          expDimRef = dataDim.expDim.sortedExpDimRefs()[0]
+          indirectDims[dataDim.dim] = isotopesDict[expDimRef]
+
+  resonances0 = []
+  resonances1 = []
+
+  distDim0, distDim1 = distDataDims
+  peakDims = peak.sortedPeakDims()
+
+  if peakDims[distIndices[0]].peakDimContribs and peakDims[distIndices[1]].peakDimContribs:
+
+    peakDim0 = peakDims[distIndices[0]]
+    for contrib in peakDim0.peakDimContribs:
+      resonance = contrib.resonance
+      if resonance.resonanceSet:
+        if residueRanges:
+          residue = resonance.resonanceSet.findFirstAtomSet().findFirstAtom().residue
+          if isResidueInRange(residue, residueRanges, distDim0):
+            resonances0.append( (contrib.resonance, []) )
+        else:
+          resonances0.append( (contrib.resonance, []) )
+
+    if peakDim0.dim in indirectDims and indirectDims[peakDim0.dim][1]:
+      isotopeA, isotopeB = indirectDims[peakDim0.dim]
+      chemElement = isotopeB.chemElement
+
+      for resonance, indirect in resonances0:
+
+        isotopeCode = '%d%s' % (isotopeB.massNumber, chemElement.symbol)
+
+        # Use getBoundResonances, to get from Cga to Hga* (and not also Hgb*)
+        resonancesA = set(x for x in getBoundResonances(resonance, recalculate=True)
+                          if x.isotopeCode == isotopeCode
+                          and x.resonanceSet)
+
+        # get covalently bound atomSets
+        atoms = set()
+        for atomSet in resonance.resonanceSet.atomSets:
+          atoms.update(getBoundAtoms(atomSet.findFirstAtom()))
+
+        atomSets = set(a.atomSet for a in atoms if a.atomSet and \
+                       a.chemAtom.chemElement is chemElement)
+
+        if resonancesA:
+          # remove covalently impossible resonances
+          resonanceSets = set(y for x in atomSets for y in x.resonanceSets)
+          resonancesA = set(x for x in resonancesA 
+                            if x.resonanceSet in resonanceSets)
+
+        if not resonancesA:
+          # make new resonances to fit covalent atoms.
+          for atomSet in atomSets:
+            resonanceB = nmrProject.newResonance(isotopeCode=isotopeCode)
+            assignAtomsToRes([atomSet,], resonanceB)
+            resonancesA.add(resonanceB)
+
+        indirect.extend(resonancesA)
+
+    peakDim1 = peakDims[distIndices[1]]
+    for contrib in peakDim1.peakDimContribs:
+     resonance = contrib.resonance
+     if resonance.resonanceSet and (resonance not in resonances0):
+       if residueRanges:
+         residue = resonance.resonanceSet.findFirstAtomSet().findFirstAtom().residue
+         if isResidueInRange(residue, residueRanges, distDim1):
+           resonances1.append( (contrib.resonance, []) )
+       else:
+         resonances1.append( (contrib.resonance, []) )
+
+    if peakDim1.dim in indirectDims and indirectDims[peakDim1.dim][1]:
+      isotopeA, isotopeB = indirectDims[peakDim1.dim]
+      chemElement = isotopeB.chemElement
+
+      for resonance, indirect in resonances1:
+
+        isotopeCode = '%d%s' % (isotopeB.massNumber, chemElement.symbol)
+
+        resonancesA = set(x for x in getBoundResonances(resonance, recalculate=True)
+                          if x.isotopeCode == isotopeCode
+                          and x.resonanceSet)
+
+        atoms = set()
+        for atomSet in resonance.resonanceSet.atomSets:
+          atoms.update(getBoundAtoms(atomSet.findFirstAtom()))
+
+        atomSets = set(a.atomSet for a in atoms if a.atomSet and \
+                       a.chemAtom.chemElement is chemElement)
+
+        if resonancesA:
+          # remove covalently impossible resonances
+          resonanceSets = set(y for x in atomSets for y in x.resonanceSets)
+          resonancesA = set(x for x in resonancesA 
+                            if x.resonanceSet in resonanceSets)
+
+        if not resonancesA:  
+          for atomSet in atomSets:
+            resonanceB = nmrProject.newResonance(isotopeCode=isotopeCode)
+            assignAtomsToRes([atomSet,], resonanceB)
+            resonancesA.add(resonanceB)
+
+        indirect.extend(resonancesA)       
+
+  return resonances0, resonances1
+
 
 def makeDistConstraints(peakList, constraintSet=None, intensityType='volume',
                         distanceFunction=None, normalise=True, labelling=None,
-                        params=None, residueRanges=None, minMerit=0.0, scale=None):
+                        params=None, residueRanges=None, minMerit=0.0, scale=None,
+                        scaleDict=None):
   """
   Makes a constraint list with constraints based upon the assigned peaks within
   a NOESY peak list. Constraints will be put in a new NMR constraint head object
@@ -2085,12 +2270,16 @@ def makeDistConstraints(peakList, constraintSet=None, intensityType='volume',
   List of Floats (distance function parameters),
   List of (List of Nmr.DataDims, Nmr.Chain, Integer, Integer),
   Float (Nmr.Peak.figOfMerit), Float (baseline intensity) 
+  Dict of Nmr.Peak --> Float (peak-specific scale)
   
   .. describe:: Output
   
   NmrConstraint.DistanceConstraintList
   """
   
+  if not scaleDict:
+    scaleDict = {}
+
   spectrum = peakList.dataSource
   #peaks  = peakList.peaks
   experiment = spectrum.experiment
@@ -2120,7 +2309,6 @@ def makeDistConstraints(peakList, constraintSet=None, intensityType='volume',
   #  workingPeaks.append(peak)
   workingPeaks = [x for x in peakList.sortedPeaks() if x.figOfMerit >= minMerit]
   
-  # Always need below to set up the peak.peakIntensity
   mean = getMeanPeakIntensity(workingPeaks, intensityType=intensityType)
   if scale: # neither Zero nor None
     mean = scale
@@ -2156,109 +2344,21 @@ def makeDistConstraints(peakList, constraintSet=None, intensityType='volume',
   
   
   for peak in workingPeaks:
-    
-    peakDims = peak.sortedPeakDims()
-    intensity = peak.findFirstPeakIntensity(intensityType=intensityType)
-    if not intensity:
-      continue
+
     if peak.figOfMerit < minMerit:
       continue
 
-    if peakDims[distIndices[0]].peakDimContribs and peakDims[distIndices[1]].peakDimContribs:
-      
-      resonances0 = []
+    intensity = peak.findFirstPeakIntensity(intensityType=intensityType)
+    if not intensity:
+      continue
+    intensityValue = abs(intensity.value)
+
+    resonances0, resonances1 = findPeakConstraintPairs(peak, residueRanges, distDataDims, distIndices, indirectDims)
+    if resonances0 and resonances1:
+
+      peakDims = peak.sortedPeakDims()
       peakDim0 = peakDims[distIndices[0]]
-      for contrib in peakDim0.peakDimContribs:
-        resonance = contrib.resonance
-        if resonance.resonanceSet:
-          if residueRanges:
-            residue = resonance.resonanceSet.findFirstAtomSet().findFirstAtom().residue
-            if isResidueInRange(residue, residueRanges, distDim0):
-              resonances0.append( (contrib.resonance, []) )
-          else:
-            resonances0.append( (contrib.resonance, []) )
-      
-      if peakDim0.dim in indirectDims and indirectDims[peakDim0.dim][1]:
-        isotopeA, isotopeB = indirectDims[peakDim0.dim]
-        chemElement = isotopeB.chemElement
-        
-        for resonance, indirect in resonances0:
-          
-          isotopeCode = '%d%s' % (isotopeB.massNumber, chemElement.symbol)
-          
-          # Use getBoundResonances, to get from Cga to Hga* (and not also Hgb*)
-          resonancesA = set(x for x in getBoundResonances(resonance, recalculate=True)
-                            if x.isotopeCode == isotopeCode
-                            and x.resonanceSet)
-          
-          # get covalently bound atomSets
-          atoms = set()
-          for atomSet in resonance.resonanceSet.atomSets:
-            atoms.update(getBoundAtoms(atomSet.findFirstAtom()))
-          
-          atomSets = set(a.atomSet for a in atoms if a.atomSet and \
-                         a.chemAtom.chemElement is chemElement)
-                         
-          if resonancesA:
-            # remove covalently impossible resonances
-            resonanceSets = set(y for x in atomSets for y in x.resonanceSets)
-            resonancesA = set(x for x in resonancesA 
-                              if x.resonanceSet in resonanceSets)
-            
-          if not resonancesA:
-            # make new resonances to fit covalent atoms.
-            for atomSet in atomSets:
-              resonanceB = nmrProject.newResonance(isotopeCode=isotopeCode)
-              assignAtomsToRes([atomSet,], resonanceB)
-              resonancesA.add(resonanceB)
-          
-          
-          indirect.extend(resonancesA)
-          
-      resonances1 = [] 
       peakDim1 = peakDims[distIndices[1]]
-      for contrib in peakDim1.peakDimContribs:
-       resonance = contrib.resonance
-       if resonance.resonanceSet and (resonance not in resonances0):
-         if residueRanges:
-           residue = resonance.resonanceSet.findFirstAtomSet().findFirstAtom().residue
-           if isResidueInRange(residue, residueRanges, distDim1):
-             resonances1.append( (contrib.resonance, []) )
-         else:
-           resonances1.append( (contrib.resonance, []) )
-           
-      if peakDim1.dim in indirectDims and indirectDims[peakDim1.dim][1]:
-        isotopeA, isotopeB = indirectDims[peakDim1.dim]
-        chemElement = isotopeB.chemElement
-        
-        for resonance, indirect in resonances1:
-          
-          isotopeCode = '%d%s' % (isotopeB.massNumber, chemElement.symbol)
-          
-          resonancesA = set(x for x in getBoundResonances(resonance, recalculate=True)
-                            if x.isotopeCode == isotopeCode
-                            and x.resonanceSet)
-            
-          atoms = set()
-          for atomSet in resonance.resonanceSet.atomSets:
-            atoms.update(getBoundAtoms(atomSet.findFirstAtom()))
-          
-          atomSets = set(a.atomSet for a in atoms if a.atomSet and \
-                         a.chemAtom.chemElement is chemElement)
-                         
-          if resonancesA:
-            # remove covalently impossible resonances
-            resonanceSets = set(y for x in atomSets for y in x.resonanceSets)
-            resonancesA = set(x for x in resonancesA 
-                              if x.resonanceSet in resonanceSets)
-          
-          if not resonancesA:  
-            for atomSet in atomSets:
-              resonanceB = nmrProject.newResonance(isotopeCode=isotopeCode)
-              assignAtomsToRes([atomSet,], resonanceB)
-              resonancesA.add(resonanceB)
-          
-          indirect.extend(resonancesA)       
 
       # Filter by correlated contributions
       contribFilter = set()
@@ -2270,100 +2370,46 @@ def makeDistConstraints(peakList, constraintSet=None, intensityType='volume',
               if contrib1.peakDim is peakDim1:
                 resonancesF = (contrib0.resonance, contrib1.resonance)
                 contribFilter.add(resonancesF)
-                
-      if resonances0 and resonances1:
-        intensity = peak.intensityValue/mean
-        
-        if normalise:
-          isoCorr0 = 0.0
-          isoCorr1 = 0.0
 
-          weight0 = 1.0/len(resonances0) 
-          weight1 = 1.0/len(resonances1) 
-          
-          for resonance, indirect in resonances0:
-            resonanceSet = resonance.resonanceSet
-            
-            propAtoms = 1.0 
-            
-            if indirect:
-              propAtoms = 0.0
-              for resonanceB in indirect:
-                if labelling:
-                  frac = max(0.1, getResonancePairLabellingFraction(resonance, resonanceB, labelling) or 1.0)
-                else:
-                  frac = 1.0  
-                
-                propAtoms += frac
+      peakMean = scaleDict.get(peak)
+      if peakMean is None:
+        if peak in scaleDict: # in this case the peak is in the dict but with a value of None
+          continue # and that means that we should skip this peak
+        peakMean = mean
+      (dist,minDist,maxDist) = getDistMinMax(intensityValue, peakMean, resonances0, resonances1, distanceFunction, normalise=normalise, labelling=labelling)
+      error = abs(maxDist - minDist)
 
-            elif labelling:
-              propAtoms *= max(0.1, getResonanceLabellingFraction(resonance, labelling) or 1.0)
- 
-            fac = weight0/propAtoms
-            isoCorr0 +=  fac
-            
-          for resonance, indirect in resonances1:
-            resonanceSet = resonance.resonanceSet
-            
-            propAtoms = 1.0 
-              
-            if indirect:
-              propAtoms = 0.0
-              for resonanceB in indirect:
-                if labelling:
-                  frac = max(0.1, getResonancePairLabellingFraction(resonance, resonanceB, labelling) or 1.0)
-                else:
-                  frac = 1.0  
-                
-                propAtoms += frac
+      fResonancePairs = set()                                     
+      for resonance0, indirect0 in resonances0:
+        resonances0 = indirect0 or [resonance0,]
+        fixedResonances0 = [getFixedResonance(constraintSet,r) for r in resonances0]
 
-            elif labelling:
-              propAtoms *= max(0.1, getResonanceLabellingFraction(resonance, labelling) or 1.0)
+        for resonance1, indirect1 in resonances1:
+          if contribFilter:
+            if (resonance0, resonance1) not in contribFilter:
+              continue
 
-            fac = weight1/propAtoms
-            isoCorr1 += fac
-          
-        else:
-          isoCorr0 = 1
-          isoCorr1 = 1
-          
-        
-        (dist,minDist,maxDist) = distanceFunction(intensity*isoCorr0*isoCorr1)
-        error = abs(maxDist - minDist)
-        
-        fResonancePairs = set()                                     
-        for resonance0, indirect0 in resonances0:
-          resonances0 = indirect0 or [resonance0,]
-          fixedResonances0 = [getFixedResonance(constraintSet,r) for r in resonances0]
-          
-          for resonance1, indirect1 in resonances1:
-            if contribFilter:
-              if (resonance0, resonance1) not in contribFilter:
-                continue
-           
-            resonances1A = indirect1 or [resonance1,]
-            fixedResonances1 = [getFixedResonance(constraintSet,r) for r in resonances1A]
-                
-            for fixedRes0 in fixedResonances0:
-              for fixedRes1 in fixedResonances1:
-                if fixedRes0 is not fixedRes1:
-                  fResonancePairs.add(frozenset([fixedRes0, fixedRes1]))
-        
-        if fResonancePairs:
-          # Otherwise you generate restraints between nothing and nothing
-        
-          constraint  = newConstraint(weight=1.0, origData=intensity, targetValue=dist,
-                                      upperLimit=maxDist, lowerLimit=minDist, error=error)
+          resonances1A = indirect1 or [resonance1,]
+          fixedResonances1 = [getFixedResonance(constraintSet,r) for r in resonances1A]
 
-          constraint.newConstraintPeakContrib(experimentSerial=experiment.serial,
-                                              dataSourceSerial=spectrum.serial,
-                                              peakListSerial=peakList.serial,
-                                              peakSerial=peak.serial)
- 
-          for fixedRes0, fixedRes1 in fResonancePairs:
-            constraint.newDistanceConstraintItem(resonances=[fixedRes0,fixedRes1])
-            
-    del peak.intensityValue
+          for fixedRes0 in fixedResonances0:
+            for fixedRes1 in fixedResonances1:
+              if fixedRes0 is not fixedRes1:
+                fResonancePairs.add(frozenset([fixedRes0, fixedRes1]))
+
+      if fResonancePairs:
+        # Otherwise you generate restraints between nothing and nothing
+
+        constraint  = newConstraint(weight=1.0, origData=intensityValue, targetValue=dist,
+                                    upperLimit=maxDist, lowerLimit=minDist, error=error)
+
+        constraint.newConstraintPeakContrib(experimentSerial=experiment.serial,
+                                            dataSourceSerial=spectrum.serial,
+                                            peakListSerial=peakList.serial,
+                                            peakSerial=peak.serial)
+
+        for fixedRes0, fixedRes1 in fResonancePairs:
+          constraint.newDistanceConstraintItem(resonances=[fixedRes0,fixedRes1])
 
   return distConstraintList
 
@@ -2538,7 +2584,6 @@ def makeAmbigDistConstraints(peakList, tolerances, chemShiftRanges, constraintSe
 
     workingPeaks.append(peak)
     
-  # Always need below to set up the peak.peakIntensity
   mean = getMeanPeakIntensity(workingPeaks, intensityType=intensityType)
   if scale: # neither Zero nor None
     mean = scale
@@ -2561,7 +2606,12 @@ def makeAmbigDistConstraints(peakList, tolerances, chemShiftRanges, constraintSe
     
     if progressBar:
       progressBar.increment()
-      
+
+    intensity = peak.findFirstPeakIntensity(intensityType=intensityType)
+    if not intensity:
+      continue
+    intensityValue = abs(intensity.value)
+
     outOfRange = 0
        
     unassignedPeakDims = []
@@ -2792,66 +2842,11 @@ def makeAmbigDistConstraints(peakList, tolerances, chemShiftRanges, constraintSe
           distalPeaks.append(peak)
     
       elif not testOnly:
-        intensity = peak.intensityValue/mean
-        
-        isoCorr0 = 0.0
-        isoCorr1 = 0.0
-
-        weight0 = 1.0/len(peakResonances[0]) 
-        weight1 = 1.0/len(peakResonances[1]) 
-        
-        for resonance, bound, indirect in peakResonances[0]:
-          resonanceSet = resonance.resonanceSet
-          propAtoms = 1.0
-          
-          if indirect:
-            propAtoms = 0.0
-            for resonanceB in indirect:
-              if labelling:
-                frac = max(0.1, getResonancePairLabellingFraction(resonance, resonanceB, labelling) or 1.0)
-              else:
-                frac = 1.0  
-              
-              propAtoms += frac
-
-          elif labelling:
-            # If an atom is only half labelled then the peak
-            # intensity is half what it would be, so the value
-            # used in the distance estimation is doubled
-            #
-            # This correction currently has limits in case of wrong labelling scheme
-            # Limits may be removed on advent of LabeledMolecule
-            propAtoms *= max(0.1, getResonanceLabellingFraction(resonance, labelling) or 1.0)
-
-          fac = weight0/propAtoms
-            
-          isoCorr0 +=  fac
-          
-        for resonance, bound, indirect in peakResonances[1]:
-          resonanceSet = resonance.resonanceSet
-          propAtoms = 1.0
-            
-          if indirect:
-            propAtoms = 0.0
-            for resonanceB in indirect:
-              if labelling:
-                frac = max(0.1, getResonancePairLabellingFraction(resonance, resonanceB, labelling) or 1.0)
-              else:
-                frac = 1.0  
-              
-              propAtoms += frac
-
-          elif labelling:
-            propAtoms *= max(0.1, getResonanceLabellingFraction(resonance, labelling) or 1.0)
-
-          fac = weight1/propAtoms
- 
-          isoCorr1 += fac
-        
-        (dist,minDistL,maxDistL) = distanceFunction(intensity*isoCorr0*isoCorr1)
-        
-        error = maxDistL - minDistL
-        constraint  = newConstraint(weight=1.0, origData=intensity,
+        resonances0 = [(resonance,indirect) for resonance, bound, indirect in peakResonances[0]]
+        resonances1 = [(resonance,indirect) for resonance, bound, indirect in peakResonances[1]]
+        (dist,minDistL,maxDistL) = getDistMinMax(intensityValue, mean, resonances0, resonances1, distanceFunction, labelling=labelling)
+        error = abs(maxDistL - minDistL)
+        constraint  = newConstraint(weight=1.0, origData=intensityValue,
                                     targetValue=dist, upperLimit=maxDistL,
                                     lowerLimit=minDistL, error=error)
         
@@ -2870,8 +2865,40 @@ def makeAmbigDistConstraints(peakList, tolerances, chemShiftRanges, constraintSe
     else:
       unmatchedPeaks.append(peak)
 
-    del peak.intensityValue
-    
-    
   return distConstraintList
 
+def transferCovalentlyBound(sourceResonance, targetResonance):
+  """
+  Set targetResonance.covalentlyBound to match sourceResonance.covalentlyBound
+  Uses serial for Resonance, and resonanceSerial for FixedResonance
+  
+  .. describe:: Input
+  
+  Nmr.Resonance or NmrConstraint.FixedResonance (twice)
+  
+  .. describe:: Output
+  
+  None
+  """
+  
+  if sourceResonance.className == 'FixedResonance':
+    covalentSerials = set(x.resonanceSerial 
+                       for x in sourceResonance.covalentlyBound)
+    if None in covalentSerials:
+      covalentSerials.remove(None)
+  else:
+    # normal resonance, make sure we update status
+    covalentSerials = set(x.serial for x in getBoundResonances(sourceResonance,
+                                                            recalculate=True))
+  
+  topObj = targetResonance.topObject
+  if targetResonance.className == 'FixedResonance':
+    boundRes = set(topObj.findFirstFixedResonance(resonanceSerial=x) 
+                   for x in covalentSerials)
+  else:
+    boundRes = set(topObj.findFirstResonance(serial=x) 
+                   for x in covalentSerials)
+  if None in boundRes:
+    boundRes.remove(None)
+    
+  targetResonance.covalentlyBound = boundRes
