@@ -378,7 +378,7 @@ def makeRatesList(rateType, experimentOrSeries, specificType, peakGroups, values
                                           resonance=resonances[i], peaks=peakGroups[i])
         
   elif rateType == 'T1rho':
-    measurementList = nmrProject.newT1rhoList(unit=unit, sf=sf,
+    measurementList = nmrProject.newT1RhoList(unit=unit, sf=sf,
                                               coherenceType=specificType,
                                               experiments=experiments)
     # wb104: 5 Apr 2011: spelling typo in data model
