@@ -44,7 +44,7 @@ from math import sqrt, log
 from ccpnmr.analysis.core.AssignmentBasic  import getResonanceResidue, makeResonanceGuiName, getResonanceAtomTuple
 from ccpnmr.analysis.core.AssignmentBasic  import updateAllShifts, getBoundResonances, getAtomSetShifts
 from ccpnmr.analysis.core.ChemicalShiftBasic import getChemAtomNmrRef, lookupAtomProbability
-from ccpnmr.analysis.core.PeakBasic        import getPeakVolume, getPeakHeight
+from ccpnmr.analysis.core.PeakBasic        import getPeakVolume, getPeakHeight, findSymmetryPeaks, searchPeaks, getClosestPeak, getPeaksOverlapScore, findClosePeaks
 from ccpnmr.analysis.core.ExperimentBasic  import getOnebondDataDims, getThroughSpacePeakLists
 from ccpnmr.analysis.core.ExperimentBasic  import getThroughSpaceDataDims, initExpTransfers
 from ccpnmr.analysis.core.MoleculeBasic    import areAtomsBound, getResidueCode
@@ -138,19 +138,20 @@ def analysePeaks(peakList):
     for peakDim in peak.sortedPeakDims():
       
       maxDelta = None
-      isotope  = None
+      isotopeCode  = None
       
       if shiftList:
         for contrib in peakDim.peakDimContribs:
-          shift = contrib.resonance.findFirstShift(parentList=shiftList)
+          resonance = contrib.resonance
+          isotopeCode = resonance.isotopeCode
+          shift = resonance.findFirstShift(parentList=shiftList)
           if shift:
-            isotope = shift.resonance.isotopeCode
             delta = abs(peakDim.realValue - shift.value)
             
             if (maxDelta is None) or (delta > maxDelta):
               maxDelta = delta
       
-      maxDeltas.append([maxDelta,isotope])  
+      maxDeltas.append([maxDelta,isotopeCode])  
       
       for contrib in peakDim.peakDimContribs:
         resonance = contrib.resonance
@@ -188,30 +189,63 @@ def analysePeaks(peakList):
     
         if peakDim.peakDimContribs:
           if peakDim2.peakDimContribs:
+            
+            # Count number of atomSets to see if result makes sense
+            atomSets1 = set()
+            atomSets2 = set()
+            for contrib2 in peakDim2.peakDimContribs:
+              resonanceSet = contrib2.resonance.resonanceSet
+              if resonanceSet:
+                for atomSet in resonanceSet.atomSets:
+                  atomSets2.add(atomSet)
+          
             for contrib in peakDim.peakDimContribs:
               resonance = contrib.resonance
+              resonanceSet = resonance.resonanceSet
               name = makeResonanceGuiName(resonance)
               bound = getBoundResonances(resonance, recalculate=True)
               impossibles  = []
+              
+              if resonanceSet:
+                for atomSet in resonanceSet.atomSets:
+                  atomSets1.add(atomSet)
 
               if bound:
+        
+                nBound = len(bound)
+                
+                for reson in bound:
+                  # Correct for multiatom atomSets (CH3 resonances e,g,)
+                  resSet = reson.resonanceSet
+                  if resSet:
+                    ll = [1]
+                    for atomSet in resSet.atomSets:
+                      length = len(atomSet.atoms)
+                      cas = atomSet.findFirstAtom().chemAtom.chemAtomSet
+                      if not cas or cas.isEquivalent is not None:
+                        # Test excludes e.g. Tyr and Phe side chains
+                        # that otherwise give spurious errors
+                        ll.append(length)
+                    # Add additional number of atoms for each bound resonance
+                    nBound += min(ll) - 1
+                    
+                if isotopeCode in ('1H','2H','19F'):
+                  if nBound > 1:
+                    assignErrors['%s mutiple %s bonds' % (name,isotopeCode)] = True
+ 
+                elif resonanceSet:
+                  chemAtom = resonance.resonanceSet.findFirstAtomSet().findFirstAtom().chemAtom
+                  if nBound > len(chemAtom.chemBonds):
+                    assignErrors['%s excessive bonds' % name] = True
+                    
                 okAtoms = False
                 foundBound = False
                 for contrib2 in peakDim2.peakDimContribs:
                   resonance2 = contrib2.resonance
+                  
                   if resonance2 in bound:
-                    foundBound = True
-                    resonanceSet = resonance.resonanceSet
                     resonanceSet2 = resonance2.resonanceSet
-                    if resonance.isotopeCode in ('1H','2H'):
-                      if len(bound) > 1:
-                        assignErrors['%s mutiple 1H bonds' % name] = True
- 
-                    elif resonanceSet:
-                      chemAtom = resonance.resonanceSet.findFirstAtomSet().findFirstAtom().chemAtom
-                      if len(bound) > len(chemAtom.chemBonds):
-                        assignErrors['%s excessive bonds' % name] = True
-                        
+                    foundBound = True
                     if resonanceSet and resonanceSet2:
                       if not areResonanceSetAtomsBound(resonanceSet, resonanceSet2):
                         names = [name,makeResonanceGuiName(resonance2)]
@@ -227,7 +261,11 @@ def analysePeaks(peakList):
                 
                 if not foundBound:
                   assignErrors['%s no bound partner' % name] = True
-                    
+            
+            if isotopeCode in ('1H','2H','19F'):
+              if len(atomSets2) > len(atomSets1):
+                assignErrors['Only %s H to bind %s Non-H' % (len(atomSets1), len(atomSets2))] = True
+                
           else:
             assignErrors['Dim %d empty' % peakDim2.dataDim.dim] = True
         
@@ -239,20 +277,25 @@ def analysePeaks(peakList):
     annotation = ' '.join([pd.annotation or '-' for pd in sortedPeakDims])
     
     locations = []
-    for peakDim in sortedPeakDims:
-      if peakDim.value is None:
-        if peakDim.position is None:
+    for pd in sortedPeakDims:
+      if pd.value is None:
+        if pd.position is None:
           locations.append('Error')
         else:
-          locations.append('%.2f' % peakDim.position)
+          locations.append('%.2f' % pd.position)
           
       else:
-        locations.append('%.2f' % peakDim.value)
+        locations.append('%.2f' % pd.value)
     
     location   = ' '.join(locations)
+
+    nearestPeak, nearestDistance = findClosestOtherPeak(peak, scale=2.0)
+    symmetryPeaks = findSymmetryPeaks(peak)
+
     datum = [peak.serial,annotation,assignErrors.keys(),location,
              volume,deltaLogVolume,
-             height,deltaLogHeight,maxDeltas]
+             height,deltaLogHeight,maxDeltas,
+             nearestPeak, nearestDistance, symmetryPeaks]
     data.append([peak, datum])
 
   posProp = 1.0
@@ -261,6 +304,25 @@ def analysePeaks(peakList):
 
   return posProp, volumeSd, heightSd, data
 
+def findClosestOtherPeak(peak, scale=1.0):
+
+  peakList = peak.peakList
+  spectrum = peakList.dataSource
+  analysisSpectrum = spectrum.analysisSpectrum
+  tolerances = spectrum.numDim * [0]
+  for analysisDataDim in analysisSpectrum.analysisDataDims:
+    dim = analysisDataDim.dataDim.dim
+    tolerances[dim-1] = analysisDataDim.assignTolerance
+
+  peaks = findClosePeaks(peak, peakList, tolerances)
+  peak2 = getClosestPeak(peak, peaks, tolerances)
+  if peak2:
+    dist = getPeaksOverlapScore(peak, peak2, tolerances)
+  else:
+    dist = None
+
+  return peak2, dist
+
 def areResonanceSetAtomsBound(resonanceSet1, resonanceSet2):
 
   for atomSet in resonanceSet1.atomSets:
@@ -335,13 +397,31 @@ def analyseChemicalShifts(shiftList):
       boundWarn = False      
       bound = getBoundResonances(resonance, recalculate=True)
       if bound:
-        if resonance.isotopeCode in ('1H','2H'):
-          if len(bound) > 1:
+        
+        nBound = len(bound)
+        
+        for reson in bound:
+          # Correct for multiatom atomSets (CH3 resonances e,g,)
+          resSet = reson.resonanceSet
+          if resSet:
+            ll = [1]
+            for atomSet in resSet.atomSets:
+              length = len(atomSet.atoms)
+              cas = atomSet.findFirstAtom().chemAtom.chemAtomSet
+              if not cas or cas.isEquivalent is not None:
+                # Test excludes e.g. Tyr and Phe side chains 
+                # that otherwise give spurious errors
+                ll.append(length)
+            # Add additional number of atoms for each bound resonance
+            nBound += min(ll) - 1
+          
+        if resonance.isotopeCode in ('1H','2H','19F'):
+          if nBound > 1:
             boundWarn = True
  
         elif resonanceSet:
           chemAtom = resonance.resonanceSet.findFirstAtomSet().findFirstAtom().chemAtom
-          if len(bound) > len(chemAtom.chemBonds):
+          if nBound > len(chemAtom.chemBonds):
             boundWarn = True
     
         if resonanceSet and not boundWarn:
