@@ -882,6 +882,8 @@ def createSpectrumWindow(project, name, axisTypes, spectrum=None,
           
           else:
             region = (0.5, 1.5) # show first plane only be default
+        elif axisType.name == 'sampled':
+          region = (0.5, 1.5) # show first plane only be default
  
       Util.createAxisPanel(windowPane, axisType, region=region)
 
@@ -940,7 +942,8 @@ def getActiveWindows(project):
   
   analysisProject = project.currentAnalysisProject
 
-  return [ w for w in analysisProject.sortedSpectrumWindows() if isActiveWindow(w) ]
+  #return [ w for w in analysisProject.sortedSpectrumWindows() if isActiveWindow(w) ]
+  return sorted([ w for w in analysisProject.spectrumWindows if isActiveWindow(w) ], key=lambda window: window.name)
 
 def getEquivalentWindows(queryWindow):
   """
