@@ -50,17 +50,20 @@ from memops.gui.IntEntry import IntEntry
 from memops.gui.LabelFrame import LabelFrame
 from memops.gui.Label import Label
 from memops.gui.LabelDivider import LabelDivider
-from memops.gui.MessageReporter import showOkCancel, showWarning
+from memops.gui.MessageReporter import showOkCancel, showWarning, showMulti
 from memops.gui.ProgressBar import ProgressBar
 from memops.gui.PulldownList import PulldownList
+from memops.gui.RadioButtons import RadioButtons
 from memops.gui.ScrolledMatrix import ScrolledMatrix 
 from memops.gui.TabbedFrame import TabbedFrame
 
-from ccpnmr.analysis.core.AssignmentBasic import getShiftLists
+from ccpnmr.analysis.core.AssignmentBasic import getShiftLists, getPeakDimResonances
 from ccpnmr.analysis.frames.NoeDistParamsFrame import NoeDistParamsFrame
 from ccpnmr.analysis.popups.BasePopup import BasePopup
 from ccpnmr.analysis.core.ConstraintBasic import makeAmbigDistConstraints, makeDistConstraints
-from ccpnmr.analysis.core.ExperimentBasic import getSpectrumIsotopes, getThroughSpacePeakLists, getNoesyPeakLists, getOnebondDataDims
+from ccpnmr.analysis.core.ConstraintBasic import findPeakConstraintPairs, getDistMinMax, getNoeDistance
+from ccpnmr.analysis.core.ExperimentBasic import getSpectrumIsotopes, getThroughSpacePeakLists, getNoesyPeakLists, getOnebondDataDims, getCompatibleHsqcPeakLists, getDiagonalBondedDataDims
+from ccpnmr.analysis.core.PeakBasic import getPeakHeight, getPeakVolume, getSimplePeakAnnotation
 from ccpnmr.analysis.core.Util import setSpectrumNoeShiftRanges, getSpectrumNoesShiftRanges
 from ccpnmr.analysis.core.Util import setSpectrumNoeResidueRanges, getSpectrumNoesResidueRanges
 from ccpnmr.analysis.core.Util import getAnalysisDataDim
@@ -334,12 +337,15 @@ class CalcDistConstraintsPopup(BasePopup):
     self.netShiftList = None
     self.labellingScheme = True
     self.structure = None
+    self.diagPeakMap = {}
+    self.hsqcPeakMap = {}
+    self.whichDistanceMap = {}
 
     BasePopup.__init__(self, parent=parent, title='Structure : Make Distance Restraints', **kw)
 
   def body(self, guiFrame):
 
-    self.geometry('600x400')
+    self.geometry('800x500')
   
     guiFrame.grid_columnconfigure(1, weight=1)
     guiFrame.grid_rowconfigure(1, weight=1)
@@ -360,11 +366,18 @@ class CalcDistConstraintsPopup(BasePopup):
     self.shiftEndEntry    = FloatEntry(self, returnCallback=self.setShiftEnd, width=8)
     
 
+    frame = Frame(guiFrame, grid=(0,0))
     tipText = 'Selects which peak list is used in the generation of distance restraints'
-    label = Label(guiFrame, text='Peak List: ', grid=(0,0))
-    self.peakListPulldown = PulldownList(guiFrame, grid=(0,1), tipText=tipText,
+    label = Label(frame, text='Peak List: ', grid=(0,0))
+    self.peakListPulldown = PulldownList(frame, grid=(0,1), tipText=tipText,
                                          callback=self.changePeakList)
 
+    frame = Frame(guiFrame, grid=(0,1))
+    tipText = 'Whether simulated peak lists are included in peak lists'
+    label = Label(frame, text='  Include simulated: ', grid=(0,0))
+    self.includeSimulatedSelect = CheckButton(frame, grid=(0,1), tipText=tipText,
+                                              callback=self.updatePeakLists)
+
     buttons = UtilityButtonList(guiFrame, helpUrl=self.help_url, grid=(0,2))
     
     #tipTexts = ['General settings that specify how restraints are made',
@@ -373,7 +386,7 @@ class CalcDistConstraintsPopup(BasePopup):
     #            'Allows restraints made with shift-peak matching to use restricted chemical shift values',
     #            'Use NOE network information to make shift-peak match generation of restraints less ambiguous']
     options = ['Settings','Residue Ranges','Shift Match Tolerances',
-               'Chem Shift Ranges','Network Anchoring']
+               'Chem Shift Ranges','Network Anchoring',]
     self.tabbedFrame = TabbedFrame(guiFrame, options=options)#, tipTexts=tipTexts)
     self.tabbedFrame.grid(row=1, column=0, columnspan=3, sticky='nsew')
     optFrame, resFrame, tolFrame, shiftFrame, netFrame = self.tabbedFrame.frames
@@ -418,9 +431,75 @@ class CalcDistConstraintsPopup(BasePopup):
     self.maxDistEntry = FloatEntry(optFrame, text='20.0', grid=(1,5), width=8, tipText=tipText)
     
 
-    self.noeFrame = NoeDistParamsFrame(optFrame, self.nmrProject)
+    self.noeFrame = NoeDistParamsFrame(optFrame, self.nmrProject,
+                                       intensityTypeCallback=self.updatePeakFrameMatrix,
+                                       distanceFunctionCallback=self.updatePeakFrameMatrix)
     self.noeFrame.grid(row=2, column=0, columnspan=6, sticky='ew')
      
+    peakFrame = self.peakFrame = LabelFrame(optFrame, text='Peak Normalisation')
+    peakFrame.grid(row=3, column=0, columnspan=6, sticky='nsew')
+    peakFrame.grid_rowconfigure(1, weight=1)
+    peakFrame.grid_columnconfigure(1, weight=1)
+
+    frame2 = Frame(peakFrame)
+    frame2.grid(row=0, column=0, sticky='w')
+    frame2.grid_rowconfigure(0, weight=1)
+    frame2.grid_columnconfigure(1, weight=1)
+
+    label = Label(frame2, text='HSQC Peak List: ')
+    label.grid(row=0, column=0, sticky='e')
+
+    tipText = 'Selects which HSQC peak list is used for analysis'
+    self.hsqcPeakListPulldown = PulldownList(frame2, grid=(0,1), tipText=tipText,
+                                             callback=self.changeHsqcPeakList)
+
+    tipTexts = ['Update the table (but most changes should be recognised automatically)',
+                'Set distance category to be used for selected rows']
+    texts = ['Refresh\nTable', 'Set Distance\nTo Use...']
+    commands = [self.updatePeakFrameMatrix, self.setWhichDistance]
+    buttons = ButtonList(peakFrame, commands=commands, texts=texts, tipTexts=tipTexts)
+    buttons.grid(row=0, column=1, sticky='w')
+
+    self.whichUsePulldown = PulldownList(self, callback=self.setWhichUse)
+    tipTexts = ['The peak',
+                'The peak annotation',
+                'The peak intensity (height or volume)',
+                'The diagonal peak',
+                'The diagonal peak annotation',
+                'The diagonal peak intensity (height or volume)',
+                'The HSQC peak',
+                'The HSQC peak annotation',
+                'The HSQC peak intensity (height or volume)',
+                'The distance derived using reference intensity',
+                'The distance derived using diagonal intensity',
+                'The distance derived using HSQC intensity',
+                'Which intensity to use for normalisation']
+    headingList = ['Peak',
+                   'Peak\nAnnotation',
+                   'Distance\nTo Use',
+                   'Ref Derived\nDistance',
+                   'Diag Derived\nDistance',
+                   'HSQC Derived\nDistance',
+                   'Peak\nIntensity',
+                   'Diagonal\nPeak',
+                   'Diagonal Peak\nAnnotation',
+                   'Diagonal Peak\nIntensity',
+                   'HSQC Peak',
+                   'HSQC Peak\nAnnotation',
+                   'HSQC Peak\nIntensity']
+    n = len(headingList) - 1
+    n0 = 2
+    n1 = n - n0
+    editWidgets = n0*[None] + [self.whichUsePulldown] + n1*[None]
+    editGetCallbacks = n0*[None] + [self.getWhichUse] + n1*[None]
+    editSetCallbacks = n0*[None] + [self.setWhichUse] + n1*[None]
+    self.peaksMatrix = ScrolledMatrix(peakFrame, multiSelect=True,
+                                      headingList=headingList,
+                                      editWidgets=editWidgets, tipTexts=tipTexts,
+                                      editSetCallbacks=editSetCallbacks,
+                                      editGetCallbacks=editGetCallbacks)
+    self.peaksMatrix.grid(row=1, column=0, columnspan=2, sticky='nsew')
+
     # Res frame
  
     tipText = 'Selects which molecular chain to set sequence/residue ranges for'
@@ -577,7 +656,6 @@ class CalcDistConstraintsPopup(BasePopup):
     self.networkPeakListsMatrix.grid(row=2, column=0, sticky='nsew') 
     
   
-  
     # Outside buttons
   
     tipTexts = ['Make restraints using the assigned peaks in the selected peak list, according to the settings',
@@ -603,13 +681,14 @@ class CalcDistConstraintsPopup(BasePopup):
     self.updateStructures()
     self.updateLabellingSchemes()
     
-    
     self.updateResidueRanges()
     self.updateTolerances()
     self.updateChemShiftRanges()
     self.updateNetShiftListPulldown()
     self.updateNetPeakLists()
 
+    self.updatePeakFrame()
+
     self.administerNotifiers(self.registerNotify)
 
   def administerNotifiers(self, notifyFunc):
@@ -652,6 +731,9 @@ class CalcDistConstraintsPopup(BasePopup):
     self.updateChemShiftRanges()
     self.updateNetShiftListPulldown()
     self.updateNetPeakLists()
+    
+    self.updatePeakFrame()
+
     BasePopup.open(self)
 
   def selectNetPeakList(self, peakList, row, col):
@@ -705,7 +787,8 @@ class CalcDistConstraintsPopup(BasePopup):
     objectList = []
     colorMatrix = []
 
-    peakLists = getThroughSpacePeakLists(self.project)
+    excludeSimulated = not self.includeSimulatedSelect.get()
+    peakLists = getThroughSpacePeakLists(self.project, excludeSimulated)
     for peakList in peakLists:
       if not hasattr(peakList,'networkAnchorUse'):
         peakList.networkAnchorUse = False
@@ -1249,6 +1332,7 @@ class CalcDistConstraintsPopup(BasePopup):
     if peakList is not self.peakList:
       self.peakList = peakList
       self.noeFrame.update(self.peakList)
+      self.updatePeakFrame()
       
       spectrum = peakList.dataSource
       if spectrum is not self.spectrum:
@@ -1261,7 +1345,6 @@ class CalcDistConstraintsPopup(BasePopup):
         self.updateTolerances()
         self.chemShiftRanges = []
         self.updateChemShiftRanges()
-            
     
   def getMolSystems(self):
   
@@ -1512,6 +1595,24 @@ class CalcDistConstraintsPopup(BasePopup):
         progressBar.destroy()
       
       else:
+        if intensityType == 'height': 
+          getIntensity = getPeakHeight
+        else:
+          getIntensity = getPeakVolume
+        scaleDict = {}
+        for peak in self.peakList.peaks:
+          whichDistance = self.whichDistanceMap.get(peak)
+          if whichDistance == 'Diagonal':
+            diagPeak = self.diagPeakMap.get(peak)
+            diagIntensity = getIntensity(diagPeak)
+            scaleDict[peak] = diagIntensity
+          elif whichDistance == 'HSQC':
+            hsqcPeak = self.hsqcPeakMap.get(peak)
+            hsqcIntensity = getIntensity(hsqcPeak)
+            scaleDict[peak] = hsqcIntensity
+          elif whichDistance == 'None':
+            scaleDict[peak] = None
+ 
         constraintList = makeDistConstraints(self.peakList,
                                              constraintSet=self.constraintSet,
                                              labelling=self.labellingScheme,
@@ -1519,7 +1620,7 @@ class CalcDistConstraintsPopup(BasePopup):
                                              residueRanges=residueRanges,
                                              minMerit=minMerit,
                                              params=params,
-                                             scale=scale)
+                                             scale=scale, scaleDict=scaleDict)
       
       self.updateConstraintSets()
       
@@ -1640,7 +1741,8 @@ class CalcDistConstraintsPopup(BasePopup):
   
     index = 0
     names = []
-    peakLists = getThroughSpacePeakLists(self.project)
+    excludeSimulated = not self.includeSimulatedSelect.get()
+    peakLists = getThroughSpacePeakLists(self.project, excludeSimulated)
     peakList = self.peakList
     getName = self.getSpectrumName
       
@@ -1790,8 +1892,256 @@ class CalcDistConstraintsPopup(BasePopup):
         showWarning('Failure','No peak lists selected',parent=self)
     
     
+  def toggleIntensityChoice(self, entry):
+
+    self.updatePeakFrameMatrix() 
+    
+  def changeHsqcPeakList(self, peakList):
+
+    self.updatePeakFrameMatrix() 
+  
+  def setWhichUse(self, junk):
+
+    peak = self.peaksMatrix.currentObject
+    self.whichDistanceMap[peak] = self.whichUsePulldown.getText()
+    self.updatePeakFrameMatrix() 
+
+  def getWhichUse(self, peak):
+
+    texts = ['None']
+    if self.findPeakDistance(peak, 1.0):
+      ###intensityType = self.intensityButtons.get().lower()
+      intensityType = self.noeFrame.intensityType
+      if peak.findFirstPeakIntensity(intensityType=intensityType):
+        texts.append('Reference')
+      if self.diagPeakMap.get(peak):
+        texts.append('Diagonal')
+      if self.hsqcPeakMap.get(peak):
+        texts.append('HSQC')
+
+    objects = None
+    ind = 0
+
+    self.whichUsePulldown.setup(texts, objects, ind)
+
+  def setWhichDistance(self):
+
+    peaks = self.peaksMatrix.currentObjects
+    if not peaks:
+      msg = 'No peaks selected in table'
+      showWarning('Warning',msg, parent=self)
+      return
+
+    objects = texts = ['Reference', 'Diagonal', 'HSQC', 'Cancel']
+    whichDistance = showMulti('Distance', 'Select which distance to use', texts, objects, parent=self)
+
+    if whichDistance == 'Cancel':
+      return
+    
+    for peak in peaks:
+      if whichDistance == 'Diagonal':
+        diagPeak = self.diagPeakMap.get(peak)
+        if not diagPeak:
+          continue
+      elif whichDistance == 'HSQC':
+        hsqcPeak = self.hsqcPeakMap.get(peak)
+        if not hsqcPeak:
+          continue
+      self.whichDistanceMap[peak] = whichDistance
+ 
+    self.updatePeakFrameMatrix()
+ 
+  def updatePeakFrame(self):
+
+    self.updateHsqcPeakLists()
+    self.updatePeakFrameMatrix()
+
+  def updateHsqcPeakLists(self, *opt):
+
+    ind = 0
+    peakList = self.peakList
+    if peakList:
+      peakLists = getCompatibleHsqcPeakLists(peakList)
+      getName = self.getSpectrumName
+      names = ['%s:%s' % (getName(pl.dataSource), str(pl.serial)) for pl in peakLists]
+    else:
+      peakLists = []
+      names = []
+
+    self.hsqcPeakListPulldown.setup(names, peakLists, ind)
+
+  def calcDiagPeakMap(self, peakList):
+
+    # this calculates a map from a peak to a corresponding diagonal peak
+
+    dataDimPairs = getDiagonalBondedDataDims(peakList.dataSource)
+    peaks = peakList.peaks
+
+    self.diagPeakMap = {}
+    for dataDim1, dataDim2 in dataDimPairs:
+      dim1 = dataDim1.dim
+      dim2 = dataDim2.dim
+      dd = {}
+      for peak in peaks:
+        peakDim1 = peak.findFirstPeakDim(dim=dim1)
+        peakDim2 = peak.findFirstPeakDim(dim=dim2)
+        resonances1 = getPeakDimResonances(peakDim1)
+        resonances2 = getPeakDimResonances(peakDim2)
+        resonances = resonances1 & resonances2
+        for resonance in resonances:
+          dd[resonance] = peak
+      for peak in peaks:
+        peakDim1 = peak.findFirstPeakDim(dim=dim1)
+        resonances1 = getPeakDimResonances(peakDim1)
+        for resonance in resonances1:
+          if resonance in dd:
+            # just take first peak that works
+            # TBD: if more than one peak that works
+            # then probably should do something smarter
+            # could happen in 3D in weird situation
+            # could happen in 4D quite easily
+            self.diagPeakMap[peak] = dd[resonance]
+
+  def calcHsqcPeakMap(self, hsqcPeakList, peakList):
+
+    # this calculates a map from a peakList peak to an hsqcPeakList peak
+    # with the same resonances
+
+    # this might not be the best/fastest way to get at the mapping
+    # also ignoring which dims are onebond because it ought to work
+    # ok but not guaranteed I guess
+    numDim = hsqcPeakList.dataSource.numDim # should be 2 presumably
+    for peak in hsqcPeakList.peaks:
+      peakCountDict = {}
+      for peakDim in peak.peakDims:
+        peakSet = set()
+        for peakDimContrib in peakDim.peakDimContribs:
+          resonance = peakDimContrib.resonance
+          for peakDimContrib2 in resonance.peakDimContribs:
+            peak2 = peakDimContrib2.peakDim.peak
+            if peak2.peakList is peakList:
+              peakSet.add(peak2)
+        for peak2 in peakSet:
+          peakCountDict[peak2] = peakCountDict.get(peak2, 0) + 1
+      for peak2 in peakCountDict:
+        if peakCountDict[peak2] == numDim:
+          self.hsqcPeakMap[peak2] = peak
+
+  def findPeakDistance(self, peak, intensityScale):
+
+    ###intensityType = self.intensityButtons.get().lower()
+    intensityType = self.noeFrame.intensityType
+    intensity = peak.findFirstPeakIntensity(intensityType=intensityType)
+    if not intensity:
+      return None
+
+    intensityValue = abs(intensity.value)
+    intensityScale = abs(intensityScale)
+
+    params   = self.noeFrame.getParams()
+    distanceFunction = lambda val:getNoeDistance(val, params)
+    resonances0, resonances1 = findPeakConstraintPairs(peak)
+    if not resonances0 or not resonances1:
+      return None
+
+    (dist,minDist,maxDist) = getDistMinMax(intensityValue, intensityScale, resonances0, resonances1, distanceFunction)
+
+    return dist
+
+  def updatePeakFrameMatrix(self, *extra):
+
+    self.diagPeakMap = {}
+    self.hsqcPeakMap = {}
+    peakList = self.peakList
+    if peakList:
+      peaks = peakList.sortedPeaks()
+      self.calcDiagPeakMap(peakList)
+      hsqcPeakList = self.hsqcPeakListPulldown.getObject()
+      if hsqcPeakList:
+        self.calcHsqcPeakMap(hsqcPeakList, peakList)
+    else:
+      peaks = []
+    
+    intensityType = self.noeFrame.intensityType
+    if intensityType == 'height': 
+      getIntensity = getPeakHeight
+    else:
+      getIntensity = getPeakVolume
+
+    textMatrix = []
+    colorMatrix = []
+    distanceColIndex = {'Reference': 3, 'Diagonal': 4, 'HSQC': 5}
+    for peak in peaks:
+      texts = []
+
+      peakIntensity = getIntensity(peak)
+      intensityScale = self.noeFrame.getScale() or 1.0
+      peakDist = self.findPeakDistance(peak, intensityScale)
+
+      diagPeak = self.diagPeakMap.get(peak)
+      if diagPeak:
+        diagText = getSimplePeakAnnotation(diagPeak)
+        diagSerial = diagPeak.serial
+        diagIntensity = getIntensity(diagPeak)
+        diagDist = self.findPeakDistance(peak, diagIntensity)
+      else:
+        diagText = None
+        diagSerial = None
+        diagIntensity = None
+        diagDist = None
+
+      hsqcPeak = self.hsqcPeakMap.get(peak)
+      if hsqcPeak:
+        hsqcText = getSimplePeakAnnotation(hsqcPeak)
+        hsqcSerial = hsqcPeak.serial
+        hsqcIntensity = getIntensity(hsqcPeak)
+        hsqcDist = self.findPeakDistance(peak, hsqcIntensity)
+      else:
+        hsqcText = None
+        hsqcSerial = None
+        hsqcIntensity = None
+        hsqcDist = None
+
+      whichDistance = self.whichDistanceMap.get(peak)
+      if whichDistance == 'Reference':
+        if peakDist is None:
+          whichDistance = None
+      elif whichDistance == 'Diagonal':
+        if diagDist is None:
+          whichDistance = None
+      elif whichDistance == 'HSQC':
+        if hsqcDist is None:
+          whichDistance = None
+
+      if not whichDistance:
+        if peakDist is not None:
+          whichDistance = 'Reference'
+        elif diagDist is not None:
+          whichDistance = 'Diagonal'
+        elif hsqcDist is not None:
+          whichDistance = 'HSQC'
+        else:
+          whichDistance = 'None'
+      texts = [peak.serial, getSimplePeakAnnotation(peak),
+               whichDistance,
+               peakDist, diagDist, hsqcDist,
+               peakIntensity,
+               diagSerial, diagText, diagIntensity,
+               hsqcSerial, hsqcText, hsqcIntensity]
+      textMatrix.append(texts)
+
+      colors = len(texts) * [None]
+      if whichDistance != 'None':
+        colors[distanceColIndex[whichDistance]] = '#C0FFC0'
+      colorMatrix.append(colors)
+
+    self.peaksMatrix.update(textMatrix=textMatrix,
+                            objectList=peaks,
+                            colorMatrix=colorMatrix)
+   
   def destroy(self):
   
     self.administerNotifiers(self.unregisterNotify)
 
     BasePopup.destroy(self)
+
