@@ -41,6 +41,8 @@ Development of a Software Pipeline. Proteins 59, 687 - 696.
 """
 import re
 
+from memops.universal.BlockData import cumulativeArray, arrayOfIndex
+
 from memops.general import Implementation
 
 from memops.gui.Button import Button
@@ -65,8 +67,9 @@ from ccpnmr.analysis.core.ChemicalShiftBasic import getChemAtomNmrRef, lookupAto
 from ccpnmr.analysis.core.ExperimentBasic import getThroughSpaceDataDims, getOnebondExpDimRefs
 from ccpnmr.analysis.core import AssignmentBasic
 from ccpnmr.analysis.core.MoleculeBasic import areResonancesBound, getResidueCode
+from ccpnmr.analysis.core.PeakBasic import pickPeak
 from ccpnmr.analysis.core.UnitConverter import ppm2pnt
-from ccpnmr.analysis.core.Util import getAnalysisDataDim
+from ccpnmr.analysis.core.Util import getAnalysisDataDim, getAnalysisPeakList
 from ccpnmr.analysis.frames.ResonanceFrame import ResonanceFrame
 from ccpnmr.analysis.popups.BasePopup import BasePopup
 
@@ -322,11 +325,12 @@ class EditAssignmentPopup(BasePopup):
                 'Show any atomic connectivities on a graphical structure display. If the peak is assigned the assigned connections are used, otherwise potential possibilities are displayed',
                 'Show a table of all the peaks currently assigned to the selected resonance',
                 'Merge the selected resonance with the others assigned to the same peak dimension',
-                'Show a table of information for the selected resonance, including the shifts of all the peaks it is linked to']
+                'Show a table of information for the selected resonance, including the shifts of all the peaks it is linked to',
+                'Predict peaks based on the possible assignments in each dimension']
     commands = [self.addSpinSystem, self.showStructConnections,
-                self.showPeaks, self.mergeResonances, self.resonanceInfo]
+                self.showPeaks, self.mergeResonances, self.resonanceInfo, self.predictPeaks]
     texts    = ['Set Same\nSpin System','Show On\nStructure',
-                'Show\nPeaks','Merge\nResonances','Resonance\nInfo']
+                'Show\nPeaks','Merge\nResonances','Resonance\nInfo', 'Predict\nPeaks']
     bottomButtons = ButtonList(guiFrame, commands=commands, texts=texts,
                                tipTexts=tipTexts, grid=(row,0))
     
@@ -335,6 +339,7 @@ class EditAssignmentPopup(BasePopup):
     self.peaksButton = bottomButtons.buttons[2]
     self.mergeButton = bottomButtons.buttons[3]
     self.infoButton = bottomButtons.buttons[4]
+    self.predictButton = bottomButtons.buttons[5]
    
     self.updateStructures() # calls updateAfter - but not always (only of struc new or changed)
     self.updateLabellingSchemes()
@@ -538,6 +543,54 @@ class EditAssignmentPopup(BasePopup):
      
        self.guiParent.browseResonanceInfo(self.contrib.resonance)
   
+  def predictPeaks(self):
+
+    if not self.peak:
+      return
+
+    spectrum  = self.peak.peakList.dataSource
+    numDim = spectrum.numDim
+    shiftList = spectrum.experiment.shiftList
+    resonanceDict = {}
+    resonancesList = []
+    for resonancePanel in self.resonancePanels[:numDim]:
+      objectList = resonancePanel.scrolledMatrix.objectList
+      if not objectList or objectList[0].className != 'Resonance':
+        return
+
+      resonances = []
+      for resonance in objectList:
+        shift = resonance.findFirstShift(parent=shiftList)
+        if not shift:
+          continue
+        resonances.append(resonance)
+        resonanceDict[resonance] = shift.value
+      if not resonances:
+        return
+      
+      resonancesList.append(resonances)
+
+    peakList  = spectrum.findFirstPeakList(isSimulated=True, name='AssignPredict')
+    if not peakList:
+      peakList = spectrum.newPeakList(isSimulated=True, name='AssignPredict')
+      analysisPeakList = getAnalysisPeakList(peakList)
+      analysisPeakList.symbolStyle = '+'
+      analysisPeakList.symbolColor = '#FF0000'
+      analysisPeakList.textColor = '#BB0000'
+
+    array = [len(resonances) for resonances in resonancesList]
+    n, cumul = cumulativeArray(array)
+
+    assignResToDim = AssignmentBasic.assignResToDim
+    r = range(numDim)
+    for i in range(n):
+      indices = arrayOfIndex(i, cumul)
+      resonances = [resonancesList[j][indices[j]] for j in r]
+      values = [resonanceDict[resonance] for resonance in resonances]
+      peak = pickPeak(peakList, values, unit=shiftList.unit, doFit=False)
+      for j, peakDim in enumerate(peak.sortedPeakDims()):
+        assignResToDim(peakDim,resonances[j])
+
   def deassignOptAtom(self):
 
     resonance = self.contrib.resonance
