@@ -59,6 +59,7 @@ from memops.gui.MessageReporter import  showOkCancel, showWarning
 from memops.gui.MultiWidget import MultiWidget
 from memops.gui.PulldownList import PulldownList
 from memops.gui.ScrolledMatrix import ScrolledMatrix
+from memops.gui.ScrolledText import ScrolledText
 
 from memops.gui.ScrolledFrame import ScrolledFrame
 from memops.gui.TabbedFrame import TabbedFrame
@@ -111,7 +112,7 @@ class EditCalculationPopup(BasePopup):
   The "Calculation Runs" tab shows the various jobs that have been setup, and
   possibly run, for the different kinds of software. The user can change the
   software calculation group via the top most pulldown menu, then change the
-  "Run Number" to look though the jobs ("runs") that have been setup. New, blank
+  "Run #" to look though the jobs ("runs") that have been setup. New, blank
   or duplicated, "run" specifications can be made at any time. A specification
   may also be deleted, but a run may only be edited if it has not been used by
   the calculation; a run is a record or what was done. A completed run may
@@ -126,13 +127,16 @@ class EditCalculationPopup(BasePopup):
   appropriate for all calculations; further incentive to use the specialised 
   program interfaces when they are available, rather than this general one.
 
-  The "Run Settings" tab is filled with a table of all of the parameters that
-  have been associated with the currently selected run. A parameter will
+  The "Run Settings" tab is filled with a table of all of the input parameters
+  that have been associated with the currently selected run. A parameter will
   have an identifying code and perhaps a group to collect certain parameters
   together. Some parameters may just represent a simple value, but others
   may associate a value with a "Data Object"; a CCPN data model entity
   containing NMR information or a reference to external data held on disk.
   
+  The "Output Summary" table lists The calculation summary or overview produced 
+  by the program (if any)).
+  
   The "Output Data" table lists all of the CCPN entities that were generated or
   modified by the selected calculation job (if it were run).
 
@@ -374,13 +378,13 @@ class NmrCalcRunFrame(Frame):
     
     self.expandGrid(0,0)
     
-    options = ['Input Data','Run Settings','Output Data']
+    options = ['Input Data','Run Settings','Output Data','Summary']
       
     tabbedFrame = TabbedFrame(self, options=options, grid=(0,0), callback=self.toggleTab)
-    frameA, frameB, frameC = tabbedFrame.frames
+    frameA, frameB, frameC, frameD = tabbedFrame.frames
     self.tabbedFrame = tabbedFrame
     
-    label = Label(tabbedFrame.sideFrame, text='Run Number:',
+    label = Label(tabbedFrame.sideFrame, text='Run #:',
                   grid=(0,0), sticky='e')
                   
     tipText = 'Selects which calculation job or "run" is currently being viewed or edited'
@@ -423,9 +427,15 @@ class NmrCalcRunFrame(Frame):
       self.chainPulldown = PulldownList(subframe, callback=self.changeChain,
                                         grid=(0,3), tipText=tipText)
         
-    label = Label(subframe, text='Run Details:', grid=(0,4))
+    label = Label(subframe, text='Protocol name:', grid=(0,4))
+    tipText = 'Name of calculation protocol for the current calculation'
+    self.protocolEntry = Entry(subframe, text='', grid=(0,5),
+                              width=32, tipText=tipText)
+    self.protocolEntry.bind('<Leave>', self.changeWmsProtocol, '+')
+        
+    label = Label(subframe, text='Run Details:', grid=(0,6))
     tipText = 'User-editable textual comment for the current calculation'
-    self.detailsEntry = Entry(subframe, text='', grid=(0,5),
+    self.detailsEntry = Entry(subframe, text='', grid=(0,7),
                               width=32, tipText=tipText)
     self.detailsEntry.bind('<Leave>', self.changeDetails, '+')
 
@@ -569,11 +579,11 @@ class NmrCalcRunFrame(Frame):
       frameB.expandGrid(0,0)
       tipTexts = ['The serial number of the parameter; an internal CCPN numbering',
                   'The identifying code (one word name) of the parameter',
-                  'The group to which the parameter belongs',
-                  'Human reasable name or decription of the parameter',
-                  'The current value associated with the parameter']
+                  'Human readable name or decription of the parameter',
+                  'The current value associated with the parameter',
+                  'The data object associated with the parameter']
  
-      headingList = ['#','Code','Group','Name','Value(s)','Data Object']
+      headingList = ['#','Code','Name','Value(s)','Data Object']
       self.paramsTable = ScrolledMatrix(frameB, headingList=headingList,
                                         grid=(0,0), tipTexts=tipTexts)
    
@@ -586,11 +596,18 @@ class NmrCalcRunFrame(Frame):
                 'the serial number of the output data, within the calculation run',
                 'The name of the output data',
                 'The identity of any CPN objects that are lined to/as output data',
-                'Any parameters which relate to the output data',
-                'Any fractional weighting that was applied to the output data']
-    headingList = ['Data Type','#','Name','CCPN Data','Parameters','Weight']
+                'Any parameters which relate to the output data',]
+                #'Any fractional weighting that was applied to the output data']
+    headingList = ['Data Type','#','Name','CCPN Data','Parameters',]#'Weight']
     self.outputTable = ScrolledMatrix(frameC,headingList=headingList,
                                       grid=(0,0), tipTexts=tipTexts)
+   
+    
+   
+    # # # # # # #  O U T P U T  S U M M A R Y # # # # # # # 
+    
+    frameD.expandGrid(0,0)
+    self.outputSummary = ScrolledText(frameD, grid=(0,0))
 
     #self.update()
 
@@ -761,6 +778,7 @@ class NmrCalcRunFrame(Frame):
     self.updateRuns()
     self.updateMolSystemPulldown()
     self.updateDetails()
+    self.updateProtocol()
     
     
     if MEASUREMENT_DATA in self.inputTypes:
@@ -919,7 +937,8 @@ class NmrCalcRunFrame(Frame):
       objectList = []
  
       if run:
-        objectList = run.sortedRunParameters()
+        objectList = [x for x in run.sortedRunParameters()
+                      if x.ioRole=='input']
  
         for param in objectList:
           values = [param.booleanValue, param.intValue, param.floatValue, param.textValue]
@@ -942,7 +961,7 @@ class NmrCalcRunFrame(Frame):
           else:
             obj = None
  
-          datum = [param.serial, param.code, param.groupId, param.name, value, obj]
+          datum = [param.serial, param.code, param.name, value, obj]
           textMatrix.append(datum)
  
       self.paramsTable.update(textMatrix=textMatrix, objectList=objectList)
@@ -971,9 +990,6 @@ class NmrCalcRunFrame(Frame):
         for param in dataObj.runParameters:
           text = param.code
           
-          if param.groupId:
-            text += '%d' % param.groupId
-          
           if param.name:
             text += ' "%s"'% param.name
       
@@ -989,20 +1005,63 @@ class NmrCalcRunFrame(Frame):
 
           if param.textValue is not None:
             text += param.textValue
+          
+          params.append(text)
       
         paramsText = ', '.join(params)
         
         textDatum = [dataType, serial,
                      dataObj.name,
                      dataObjText,
-                     paramsText,
-                     dataObj.weight]
+                     paramsText,]
+                     #1.0]
+                     #dataObj.weight]  # NBNB TBD DataOBj no longer has weight. To fix properly NBNB
         
         objectList.append(dataObj)
         textMatrix.append(textDatum)
-        
     
+      outputPars = [x for x in run.sortedRunParameters()
+                    if x.ioRole=='output' and x.data is None and x.name != 'summary']
+ 
+      objectList.extend(outputPars)
+ 
+ 
+      for param in outputPars:
+        values = [param.booleanValue, param.intValue, param.floatValue, param.textValue]
+        values = [v for v in values if v is not None]
+ 
+        if not values:
+          value = None
+ 
+        elif len(values) == 1:
+          value = values[0]
+ 
+        else:
+          value = ', '.join([str(v) for v in values])
+ 
+        #datum = param.data
+        #if datum:
+        #  info = getDataObjText(datum)
+        #  obj = '%s (%s)' % (info, datum.ioRole)
+ 
+        #else:
+        #  obj = None
+ 
+        datum = ['Parameter', param.serial, param.name, value, ""]
+        textMatrix.append(datum)
+ 
     self.outputTable.update(textMatrix=textMatrix, objectList=objectList)
+    # Output Summary
+    
+    if run:
+      valueObj = run.findFirstRunParameter(ioRole='output', name='summary')
+      if valueObj:
+        self.outputSummary.setText(valueObj.textValue)
+      else:
+        self.outputSummary.setText()
+    else:
+      self.outputSummary.setText()
+    
     self.after_idle(lambda:self.configure(cursor=""))
     self.waiting = False
   
@@ -1311,11 +1370,23 @@ class NmrCalcRunFrame(Frame):
       if value != self.run.details:
         self.run.details = value
   
+  def changeWmsProtocol(self, event):
+  
+    if self.run:
+      value = self.protocolEntry.get().strip() or None
+      if value != self.run.wmsProtocolName:
+        self.run.wmsProtocolName = value
+  
   def updateDetails(self):
   
     if self.run:
       self.detailsEntry.set(self.run.details)  
   
+  def updateProtocol(self):
+  
+    if self.run:
+      self.protocolEntry.set(self.run.wmsProtocolName)  
+  
   def changeMolSystem(self, molSystem):
     
     if self.run and molSystem:
@@ -1395,21 +1466,20 @@ class NmrCalcRunFrame(Frame):
     index = 0
     molSystems = []
     
-    if self.run:
-      data = self.run.findFirstData(className='MolSystemData',
-                                    ioRole='input')
+    run = self.run
+    if run:
+      data = (run.findFirstData(className='MolSystemData', ioRole='input')
+              or run.findFirstData(className='MolResidueData', ioRole='input'))
       
       if data:
         molSystem = data.molSystem
+        molSystems = [molSystem,]
       else:
         molSystem = None
       
       if self.runIsEditable():
         molSystems = self.project.sortedMolSystems()
-
-      elif molSystem:
-        molSystems = [molSystem,]
-
+      
       if molSystems:
         names = [ms.code for ms in molSystems]
         
