@@ -616,9 +616,10 @@ class EditWindowPopup(BasePopup):
   
     which = self.which_windows.get()
     if which == ACTIVE_OPTIONS[0]:
-      windows = self.analysisProject.sortedSpectrumWindows()
+      windows = self.analysisProject.spectrumWindows
     else:
       windows = self.parent.getActiveWindows()
+    windows = sorted(windows, key=lambda window: window.name)
     
     panes = []
     for window in windows:
