@@ -1602,8 +1602,11 @@ class FollowShiftChangesPopup(BasePopup):
   
     self.updateButtons()
     
-    if self.fitGraphPopup:
-      self.showFunctionFit(noOpen=True)
+    # wb104: 26 Mar 2014: below causes exception when switch between expSeries 
+    # (that exception only happens if the functionFit is open)
+    # this means that some graphs might need to be updated manually
+    #if allowShowFunctionFit and self.fitGraphPopup:
+    #  self.showFunctionFit(noOpen=True)
     
     fitFunc = self.fitFunc
     nParams = getFitMethodInfo()[fitFunc-1][1]
@@ -1822,7 +1825,7 @@ class FollowShiftChangesPopup(BasePopup):
       self.expSeries = expSeries
       
       sampleConditions = self.getSampleConditions()
-      if sampleConditions and (sampleConditions[0].unit in CONC_FACTORS):
+      if sampleConditions and (sampleConditions[0].unit in CONC_FACTORS) and (self.unitPulldown.getText() not in CONC_FACTORS):
         self.unitPulldown.set(sampleConditions[0].unit)
       
       self.clearPeakGroups()
