@@ -57,9 +57,13 @@ from memops.gui.MessageReporter import showWarning
 
 from ccpnmr.analysis.core.QualityControlBasic import analyseAssignmentCompleteness, analyseNoeAssignments
 from ccpnmr.analysis.core.QualityControlBasic import analyseChemicalShifts, analysePeaks
-from ccpnmr.analysis.core.AssignmentBasic import getShiftLists
+from ccpnmr.analysis.core.AssignmentBasic import getShiftLists, getBoundResonances, getPeakDimResonances, getAtomResonances
 
-from ccpnmr.analysis.core.ExperimentBasic  import getThroughSpacePeakLists
+from ccpnmr.analysis.core.ExperimentBasic  import getThroughSpacePeakLists, getEquivalentDataDims, getOnebondExpDimRefs, getPrimaryExpDimRef
+
+from ccpnmr.analysis.core.PeakBasic  import findSymmetryPeaks
+
+from ccpnmr.analysis.core.MoleculeBasic import getRandomCoilShift
 
 def qualityTestMacro(argServer):
   
@@ -205,6 +209,7 @@ class ViewQualityReportsPopup(BasePopup):
     self.guiParent  = parent
     self.molSystemA = None
     self.molSystemB = None
+    self.molSystemD = None
     self.shiftListA = None
     self.shiftListC = None
     self.peakListB  = None
@@ -376,6 +381,7 @@ class ViewQualityReportsPopup(BasePopup):
     
     
     frameD.grid_rowconfigure(1, weight=1)
+    frameD.grid_rowconfigure(3, weight=1)
      
     frame = Frame(frameD)
     frame.grid(row=0, column=0,sticky='ew')
@@ -400,16 +406,38 @@ class ViewQualityReportsPopup(BasePopup):
                 'The integration volume of the peak; colour indicates deviation from the peak list average on a logarithmic scale',
                 'The standard deviation of the peak integration volume on a logarithmic scale; highlights strong/weak outliers (which could be genuine)',
                 'The height intensity of the peak; colour indicates deviation from the peak list average on a logarithmic scale',
-                'The standard deviation of the height intensity  on a logarithmic scale; highlights strong/weak outliers (which could be genuine) and inappropriate peak sign']
+                'The standard deviation of the height intensity  on a logarithmic scale; highlights strong/weak outliers (which could be genuine) and inappropriate peak sign',
+                'An indication of whether or not there is a symmetry peak in the same location and with the same assignment',
+                'The nearest other peak',
+                'The distance to the nearest other peak, an indication of overlap']
     headingList = ['#','Assignment','Assign\nErrors',
                    'Location','Volume','Log Vol\nDev',
-                   'Height','Log Height\nDev']
+                   'Height','Log Height\nDev',
+                   'Symmetry\nPeaks', 'Nearest\nPeak', 'Nearest\nDistance']
     
     self.peakMatrixTipTexts = tipTexts
     self.peakMatrix = ScrolledMatrix(frameD, headingList=headingList,
                                      multiSelect=True, callback=self.selectObject,
                                      tipTexts=tipTexts, grid=(1,0))
     
+    frame = Frame(frameD, grid=(2,0))
+    frame.grid_columnconfigure(2, weight=1)
+   
+    label = Label(frame, text='Mol System:', grid=(0,0))
+    tipText = 'Selects which molecular system (group of chains) to display assignment status information for'
+    self.molSystemPulldownD = PulldownList(frame, grid=(0,1), tipText=tipText,
+                                           callback=self.changeMolSystemD)
+
+    tipTexts = ['The row number',
+                'The chain code',] + 7*['']
+    headingList = ['#','Chain','Seq Code','Ccp Code',
+                   'Atom','Resonances','Shifts',
+                   'Delta', 'Stdev']
+    
+    self.peakAtomMatrix = ScrolledMatrix(frameD, headingList=headingList,
+                                         multiSelect=True, callback=self.selectObject,
+                                         tipTexts=tipTexts, grid=(3,0))
+    
     
     #
     # Main
@@ -421,9 +449,10 @@ class ViewQualityReportsPopup(BasePopup):
   
     tipTexts = ['Manually update the currently viewed table in light of assignment changes etc.',
                 'Show a table of the peaks that relate to the selected rows, either directly or by virtue of assignment (depending on the current table)',
+                'Show a table of the peaks and symmetry peaks that relate to the selected rows',
                 'Show a table of resonances that relate to the selected rows, either directly or by virtue of assignment (depending on the current table)']
-    texts = ['Update Table','Show Peaks','Show Resonances']
-    commands = [self.refresh, self.showObjects, self.showResonances]
+    texts = ['Update Table','Show Peaks','Show Peaks and Symmetry Peaks','Show Resonances']
+    commands = [self.refresh, self.showObjects, self.showSymmetryPeaks, self.showResonances]
     self.bottomButtons = ButtonList(guiFrame, texts=texts, tipTexts=tipTexts,
                                     commands=commands, grid=(1,0))
     self.showButton = self.bottomButtons.buttons[1]
@@ -458,7 +487,7 @@ class ViewQualityReportsPopup(BasePopup):
     commands = [self.reportAssignStatus,
                 self.reportNoe,
                 self.reportChemShifts,
-                self.reportPeaks]
+                self.reportPeaksAndAtoms]
                 
     commands[i]()
   
@@ -481,6 +510,10 @@ class ViewQualityReportsPopup(BasePopup):
               
               resonances.add(contrib.resonance)
         
+      elif self.object.className == 'Atom':
+        for atom in self.peakAtomMatrix.currentObjects:
+          resonances.update(getAtomResonances(atom))
+
       elif self.object.className == 'Shift':
         shiftList = self.shiftListC
         for shift in self.resonanceMatrix.currentObjects:
@@ -510,8 +543,8 @@ class ViewQualityReportsPopup(BasePopup):
       if resonances:
         self.guiParent.viewSelectedResonances(resonances, shiftList)
     
-  def showObjects(self):
-  
+  def getRelatedPeaks(self):
+
     peaks = None
     if self.object:
     
@@ -562,9 +595,25 @@ class ViewQualityReportsPopup(BasePopup):
 
         peaks = list(peaks)
 
+    return peaks
+
+  def showObjects(self):
+  
+    peaks = self.getRelatedPeaks()
     if peaks:
       self.guiParent.viewPeaks(peaks)
 
+  def showSymmetryPeaks(self):
+  
+    peaks = self.getRelatedPeaks()
+    if peaks:
+      allPeaks = []
+      for peak in peaks:
+        allPeaks.append(peak)
+        symmetryPeaks = findSymmetryPeaks(peak)
+        allPeaks.extend(symmetryPeaks)
+      self.guiParent.viewPeaks(allPeaks)
+
   def selectObject(self, object, row, col):
   
     self.object = object
@@ -777,6 +826,82 @@ class ViewQualityReportsPopup(BasePopup):
                                 objectList=objectList,
                                 colorMatrix=colorMatrix)  
 
+  def commonResonances(self, peakDim1, peakDim2):
+
+    # say it is the same if any resonance is in both (a bit liberal)
+
+    resonances1 = getPeakDimResonances(peakDim1)
+    resonances2 = getPeakDimResonances(peakDim2)
+
+    return resonances1 & resonances2
+
+  def boundResonancesOfCorrectIsotopeCode(self, peak, dataDim, isotopeCode):
+
+    peakDim = peak.findFirstPeakDim(dataDim=dataDim)
+    resonances = getPeakDimResonances(peakDim)
+    resonanceSet = set()
+    for resonance in resonances:
+      boundResonances = getBoundResonances(resonance)
+      resonanceSet.update([boundResonance for boundResonance in boundResonances if isotopeCode == boundResonance.isotopeCode])
+
+    return resonanceSet
+
+  def determineSymmetryPeakColor(self, dataDimPairs, peak, symmetryPeaks):
+
+    if not dataDimPairs:
+      return None
+
+    pairs = []
+
+    for dataDim1, dataDim2, isotopeCode1, isotopeCode2 in dataDimPairs:
+      if isotopeCode1:
+        boundResonances = self.boundResonancesOfCorrectIsotopeCode(peak, dataDim2, isotopeCode1)
+      else: # isotopeCode2
+        boundResonances = self.boundResonancesOfCorrectIsotopeCode(peak, dataDim1, isotopeCode2)
+
+      if boundResonances: # it is worth checking for symmetry peaks
+        peakDim1 = peak.findFirstPeakDim(dataDim=dataDim1)
+        peakDim2 = peak.findFirstPeakDim(dataDim=dataDim2)
+        pairs.append((peakDim1, peakDim2, dataDim1, dataDim2))
+
+
+    if not pairs:
+      return None
+
+    if not symmetryPeaks:
+      return '#FF0000'
+
+    for peak2 in symmetryPeaks:
+      for peakDim1, peakDim2, dataDim1, dataDim2 in pairs:
+        peakDim3 = peak2.findFirstPeakDim(dataDim=dataDim1)
+        peakDim4 = peak2.findFirstPeakDim(dataDim=dataDim2)
+        if not self.commonResonances(peakDim1, peakDim4) or not self.commonResonances(peakDim2, peakDim3):
+          break
+      else:
+        #return '#a0FFa0'
+        return None
+
+    return '#FFa0a0'
+
+  def getOnebondIsotopeCode(self, dataDim):
+
+    expDim = dataDim.expDim
+    expDimRef = getPrimaryExpDimRef(expDim)
+    expDimRefPairs = getOnebondExpDimRefs(expDim.experiment)
+
+    for expDimRef1, expDimRef2 in expDimRefPairs:
+      if expDimRef1 is expDimRef:
+        return expDimRef2.isotopeCodes[0]
+      elif expDimRef2 is expDimRef:
+        return expDimRef1.isotopeCodes[0]
+
+    return None
+    
+  def reportPeaksAndAtoms(self, *event):
+
+    self.reportPeaks()
+    self.reportAtoms()
+  
   def reportPeaks(self, *event):
   
     self.showButton.enable()
@@ -792,7 +917,8 @@ class ViewQualityReportsPopup(BasePopup):
 
     headingList = ['#','Assignment','Assign\nErrors',
                    'Location','Volume','Log Vol\nDev',
-                   'Height','Log Height\nDev']
+                   'Height','Log Height\nDev',
+                   'Symmetry\nPeaks', 'Nearest\nPeak', 'Nearest\nDistance']
     
     tipTexts = self.peakMatrixTipTexts[:]
                    
@@ -803,15 +929,24 @@ class ViewQualityReportsPopup(BasePopup):
         headingList.append(u'F%d Max\nShift \u0394' % (i+1))
         tip = "For dimension %d, the maximum difference between an assigned resonance's (averaged) chemical shift value and the peak position"
         tipTexts.append(tip % (i+1))
+
+      equivDataDimPairs = getEquivalentDataDims(self.peakListD.dataSource)
+      dataDimPairs = []
+      for dataDim1, dataDim2 in equivDataDimPairs:
+        isotopeCode1 = self.getOnebondIsotopeCode(dataDim1)
+        isotopeCode2 = self.getOnebondIsotopeCode(dataDim2)
+        # no point checking if both fixed or if neither fixed
+        if (isotopeCode1 and not isotopeCode2) or (isotopeCode2 and not isotopeCode1):
+          dataDimPairs.append((dataDim1, dataDim2, isotopeCode1, isotopeCode2))
     
       posProp, volSd, htSd, data = analysePeaks(self.peakListD)
 
       for peak, datum in data:
         colors = []
-        serial,annotation,assignErrors,location,volume,volDelta,height,htDelta,maxDeltas = datum
+        serial,annotation,assignErrors,location,volume,volDelta,height,htDelta,maxDeltas, nearestPeak, nearestDistance, symmetryPeaks = datum
         alert = False
 
-        colors = [None] * 8
+        colors = [None] * 11
         maxDeltaData = []
         for maxDelta, isotope in maxDeltas:
           maxDeltaData.append(maxDelta)
@@ -874,13 +1009,31 @@ class ViewQualityReportsPopup(BasePopup):
             if volume > 0.0:
               colors[4] = '#FF0000'
          
+        colors[8] = self.determineSymmetryPeakColor(dataDimPairs, peak, symmetryPeaks)
+        if colors[8]:
+          alert = True
+
+        colors[9] = colors[10] = None
+        if nearestPeak:
+          if nearestDistance < 0.5:
+            colors[10] = '#FF0000'
+            alert = True
+          elif nearestDistance < 1.0:
+            colors[10] = '#FFa0a0'
+            alert = True
+
         if onlyAlerts and not alert:
           continue
-        
+
         datum = [serial,annotation,
                  ','.join(assignErrors),location,
                  volume, nVolSd, height, nHtSd]
+
+        datum.append(', '.join([str(pk.serial) for pk in symmetryPeaks]))
+        datum.extend([nearestPeak and nearestPeak.serial, nearestDistance])
+
         datum.extend(maxDeltaData)
+
         objectList.append(peak)
         textMatrix.append(datum)
         colorMatrix.append(colors)
@@ -891,6 +1044,53 @@ class ViewQualityReportsPopup(BasePopup):
                            colorMatrix=colorMatrix,
                            tipTexts=tipTexts)  
 
+  def reportAtoms(self, *event):
+
+    molSystem = self.molSystemD
+
+    if molSystem:
+      chains = molSystem.sortedChains()
+      shiftList = self.peakListD.dataSource.experiment.shiftList
+    else:
+      chains = []
+
+    textMatrix = []
+    objectList = []
+    n = 0
+    for chain in chains:
+      for residue in chain.sortedResidues():
+        for atom in residue.sortedAtoms():
+          if atom.name[0] not in ('H', 'C', 'N'):
+            continue
+          chemAtom = atom.chemAtom
+          resonances = sorted(list(getAtomResonances(atom)))
+          resonanceText = ', '.join(['%d' % resonance.serial for resonance in resonances])
+          if shiftList:
+            values = []
+            deltas = []
+            randomCoilShift = getRandomCoilShift(chemAtom)
+            for resonance in resonances:
+              shift = resonance.findFirstShift(parentList=shiftList)
+              if shift:
+                values.append('%.3f' % shift.value)
+                deltas.append('%.3f' % (shift.value - randomCoilShift))
+              else:
+                values.append(None)
+                deltas.append(None)
+            shiftText = ', '.join(['%s' % value for value in values])
+            deltaText = ', '.join(['%s' % delta for delta in deltas])
+          else:
+            shiftText = ''
+            deltaText = ''
+
+          datum = [n+1, chain.code, residue.seqCode, residue.ccpCode,
+                   atom.name, resonanceText, shiftText, deltaText, 0]
+          textMatrix.append(datum)
+          objectList.append(atom)
+  
+    self.peakAtomMatrix.update(textMatrix=textMatrix,
+                               objectList=objectList)
+
   def changeMolSystemA(self, molSystem):
   
     if molSystem is not self.molSystemA:
@@ -917,6 +1117,12 @@ class ViewQualityReportsPopup(BasePopup):
       self.molSystemB = molSystem
       self.refresh()
   
+  def changeMolSystemD(self, molSystem):
+  
+    if molSystem is not self.molSystemD:
+      self.molSystemD = molSystem
+      self.refresh()
+  
   def changeShiftListA(self, shiftList):
   
     if shiftList is not self.shiftListA:
@@ -991,6 +1197,36 @@ class ViewQualityReportsPopup(BasePopup):
      
     self.molSystemPulldownA.setup(names, molSystems, indexA)
     self.molSystemPulldownB.setup(names, molSystems, indexB)
+
+    self.updateMolSystemD()
+
+  def updateMolSystemD(self, *obj):
+  
+    indexD = 0
+    molSystemD = self.molSystemD
+
+    if self.peakListD:
+
+      experiment = self.peakListD.dataSource.experiment
+      molSystems = experiment.sortedMolSystems()
+
+      names  = [ms.code for ms in molSystems]
+
+    else:
+
+      molSystems = []
+      names = []
+
+    if names:
+      if molSystemD not in molSystems:
+        molSystemD = molSystems[0]
+      
+      indexD = molSystems.index(molSystemD)
+    
+    if molSystemD is not self.molSystemD:
+      self.changeMolSystemD(molSystemD)
+
+    self.molSystemPulldownD.setup(names, molSystems, indexD)
     
   def updateShiftLists(self, *obj):
   
@@ -1043,6 +1279,7 @@ class ViewQualityReportsPopup(BasePopup):
     
     self.peakListPulldownD.setup(names, peakLists, indexD)
     
+    self.updateMolSystemD()
     
   def destroy(self):
 
