@@ -865,7 +865,7 @@ IoSetupList = [
      'cyana':   ['dihedrals.aco',1,1,       [['SelectionList','version','Cyana version:',['2.0','2.1'],None]],
                                             [['SelectionList','version','Cyana version:',['2.0','2.1'],None]]],
      'nmrStar': ['nmrStar.str',1,0,         [],[]],
-     'talos':   ['dihedral.tab',1,0,        [['MultiSelectionList','validClasses','Classes for valid constraint:',['Good','New','Warn','None'],None],
+     'talos':   ['dihedral.tab',1,0,        [['MultiSelectionList','validClasses','Classes for valid constraint:',['Good','New','Warn','Dyn','Strong','Generous','None'],None],
                                              ['IntEntry','validCount','Count variable lower limit for valid constraint:',0,None],
                                              ['FloatEntry','multiplyDeviation','Multiplication factor to convert deviation in constraint range:',2.0,None]],
                                             []]},
