@@ -29,7 +29,7 @@ from memops.gui.Frame             import Frame
 from ccp.api.nmr                  import NmrCalc
 from ccp.util.NmrCalc             import setRunParameter, setRunTextParameter, getRunTextParameter
 
-from ccpnmr.analysis.core.StructureBasic import makePdbFromStructure, getStructureFromFile
+from ccp.lib.StructureIo import makePdbFromStructure, getStructureFromFile
 
 from grenoble.BlackledgeModule    import Io as moduleIo
 
