@@ -240,26 +240,7 @@ def saveProject(project, newPath = None, newProjectName = None, changeBackup = T
       # below is because of data model limit
       newProjectName = newProjectName[:32]
 
-  # change project name
-  if newProjectName != oldProjectName:
-    project.override = True # TBD: for now name is frozen so change this way
-    try:
-      # below constraint is not checked in setName() if override is True so repeat here
-      isValid = newProjectName.isalnum()  # superfluous but faster in most cases
-      if not isValid:
-        for cc in newProjectName:
-          if cc != '_' and not cc.isalnum():
-            isValid = False
-            break
-        else:
-          isValid = True
-      if (not (isValid)):
-        raise ApiError('project name must only have characters that are alphanumeric or underscore')
-
-      # below checks for length of name as well
-      project.name = newProjectName
-    finally:
-      project.override = False
+  renameProject(project, newProjectName)
 
   # if newPath same as oldPath, check if newProjectName already exists if it's not same as oldProjectName
   if newPath == oldPath:
@@ -486,6 +467,37 @@ def saveProject(project, newPath = None, newProjectName = None, changeBackup = T
 
     raise
 
+def renameProject(project, newProjectName):
+  """ Rename project.
+  """
+  
+  print '### renaming', project.name, newProjectName
+  
+  # change project name
+  if newProjectName == project.name:
+    return
+  
+  else:
+    project.override = True # TBD: for now name is frozen so change this way
+    try:
+      # below constraint is not checked in setName() if override is True so repeat here
+      isValid = newProjectName.isalnum()  # superfluous but faster in most cases
+      if not isValid:
+        for cc in newProjectName:
+          if cc != '_' and not cc.isalnum():
+            isValid = False
+            break
+        else:
+          isValid = True
+      if (not (isValid)):
+        raise ApiError('project name must only have characters that are alphanumeric or underscore')
+
+      # below checks for length of name as well
+      project.name = newProjectName
+    finally:
+      project.override = False
+
+
 def saveRepository(repository, newPath = None, createFallback = False,
                    removeExisting = False, showYesNo = None):
   """
@@ -712,6 +724,9 @@ def loadProject(path, projectName=None, showWarning=None, askFile=None,
     oldPath = repository.url.path
     warningMessages.append('Project file has moved from\n"%s"\nto\n"%s"' % (oldPath, path))
     repository.url = Implementation.Url(path=path)
+    # Necessary because activeRepositories are not set right 
+    # if file names do not match:
+    project.__dict__['activeRepositories'].append(repository)
 
   projectRepository = repository
 
@@ -1081,3 +1096,13 @@ def getCcpFileString(fileString):
   return doConvertStringToFileName(fileString, validFileNamePartChars,
                                    defaultFileNameChar)
 
+
+def findCcpnProjectDirs(inDir):
+  """ Find list of (potential) CCPN project dirs within inDir
+  """
+  result = []
+  for dirpath, dirnames, filenames in os.walk(inDir):
+    if dirpath.endswith('memops') and 'Implementation' in dirnames:
+      result.append(os.path.dirname(dirpath))
+  #
+  return result
