@@ -313,369 +313,6 @@ def copySubTree(sourceObj, newParent, maySkipCrosslinks=False,
   #
   return result
   
-
-'''
-def getTreeClassData(sourceClass, skipOtherPackages=False):
-  """ get class data for the of children rooted in sourceClass
-  OBSOLETE and BROKEN
-  """
-  
-  appDataName = ImpConstants.applicationRoleName
-  appDataRole = sourceClass.elementDict.get(appDataName)
-  doneAppData = False
-  
-  package = sourceClass.package
-  
-  result = {}
-    
-  classes = list(sourceClass.getNonabstractSubclasses())
-  classes.extend(appDataRole.otherClass.getNonabstractSubclasses())
-  
-  for clazz in classes:
- 
-    elementDict = clazz.elementDict
-    
-    if not result.has_key(clazz):
-      
-      result[clazz] = dd = getClassData(clazz)
-      
-      for ss in ('childlinks', 'singlekids'):
-        remove = []
-        for name in dd[ss]:
-          if name != appDataName or not doneAppData :
-            elem = elementDict[name]
-            otherClass = elem.otherClass
-            
-            if otherClass.package is package or not skipOtherPackages:
-              classes.extend(otherClass.getNonabstractSubclasses())
-            else:
-              # link crosses package boundaries and we skip - remove link
-              remove.append(name)
- 
-          if name == appDataName:
-            doneAppData = True
-        for name in remove:
-          dd[ss].remove(name)
-  #
-  return result
-
-
-def getClassData(clazz):
-  """ get class data summary, 
-  non-derived attributes and various kinds of links
-  OBSOLETE and BROKEN
-  """
-  
-  from memops.metamodel import MetaModel
-  
-  elementDict = clazz.elementDict
-  result = {'clazz':clazz}
-  result['keys'] = keys = []
-  result['attrs'] = attrs = []
-  for name in clazz.attributeAllNames:
-    elem = elementDict[name]
-    if not elem.isDerived:
-      attrs.append(name)
- 
-  result['crosslinks'] = crosslinks = []
-  result['childlinks'] = childlinks = []
-  result['singlekids'] = singlekids = []
-  for name in clazz.roleAllNames:
-    elem = elementDict[name]
-    if not elem.isDerived:
-      hierarchy = elem.hierarchy
-      if hierarchy == ImpConstants.parent_hierarchy:
-        result['parent'] = name
-      elif hierarchy == ImpConstants.child_hierarchy:
-        if elem.hicard ==1:
-          singlekids.append(name)
-        else:
-          childlinks.append(name)
- 
-      else:
-
-        crosslinks.append((name,elem.hicard))
-  
-  for name in clazz.keyNames:
-    elem = elementDict[name]
-    isSingle = (elem.hicard == 1)
-    doSort = False
-    if isinstance(elem,MetaModel.MetaRole):
-      isLink = True
-      if not isSingle:
-        doSort = True
-    else:
-      isLink = False
-    
-    keys.append((name,isSingle,isLink,doSort))
-     
-  #
-  return result
-  
-  
-def compareSubTrees(copyObj, origObj, acceptSubset=False, verbose=1):
-  """ Driver routine for comparing subtrees.
-  OUT-OF-DATE and BROKEN
-  """
-  
-  xx = subTreesDiffer(copyObj, origObj, acceptSubset)
-  
-  if acceptSubset:
-    ss = 'incompatible'
-    ss2 = ''
-  else:
-    ss = 'different'
-    ss2 = '   '
-  
-  if xx:
-    extras1,extras2,diffs,objMatches = xx
-    nDiffs = len(diffs)
-    
-    if verbose  > 0:
-      if verbose  > 1:
-        if extras1:
-          print "Unmatched objects in first subtree:"
-          for obj in extras1:
-            print obj
- 
-        if extras2:
-          print "Unmatched objects in second subtree:"
-          for obj in extras2:
-            print obj
- 
-      if diffs:
-             
-        if verbose > 2:
-           print "Objects %s in following elements:" % ss
-           for obj,tags in diffs.items():
-             obj2 = objMatches.get(obj)
-             print obj, obj2,':'
-             for tag in tags:
-               if tag == '__class__':
-                 print ' ', tag
-               else:
-                 print "  %s: %s | %s" % (
-                  tag, `getattr(obj,tag)`, `getattr(obj2,tag)`
-                 )
-        else:
-           print "Objects %s in following elements:" % ss
-           for obj,tags in diffs.items():
-             print obj,' : ', tags
-      
-    #if topDiffs:
-    #  print "Top object %s in following elements:" % ss
-    #  print copyObj,' : ', topDiffs
-     
-    print """
-Matching objects                   : %s
-Unmatched objects in subtree 1     : %s
-Unmatched objects in subtree 2     : %s
-Objects %s in subtrees   %s: %s
-""" % (len(objMatches)-nDiffs, len(extras1), len(extras2), ss, ss2, nDiffs)
-    
-  elif acceptSubset:
-    print "first subtree conforms to second subtree"
-    
-  else:
-    print 'subtrees compare identical'
-
-
-def subTreesDiffer(top1, top2, acceptSubset=False):
-  """ Compare memops object subtrees rooted in top1 and top2 
-  If acceptSubset is True, links that are set in the
-  top2 tree but not in the top1 tree will not be marked as errors.
-  OUT-OF-DATE and BROKEN
-  
-  """
-  
-  from memops.general.Implementation import Implementation as implClass
-  
-  from memops.api.Implementation import ApplicationData
-  
-  classData = getTreeClassData(top1.metaclass)
-  
-  tops = (top1,top2)
-  objLists = ([],[])
-  keyMaps = ({},{})
-  for ii in (0,1):
-    
-    # make dictionary with keylist for every object encountered
-    # NB to avoid having to make the key when you first see an object
-    # the code depends on setting a keyList the first time 
-    # an object is encountered, and the fillling it in later.
-    objs = objLists[ii]
-    objs.append(tops[ii])
-    keyTrackDict = keyMaps[ii]
-    #keyTrackDict[objs[0].parent] = []
-    
-    for obj in objs:
-      
-      dd = classData[obj.metaclass]
-    
-      if obj is objs[0]:
-        keyTrackDict[obj] = []
-        
-      else:
-        keyTuple = obj.__class__.keyPath[-1][-1]
- 
-        # get keylist if it is there, or set a new one
-        objKey = keyTrackDict.get(obj)
-        if objKey is None:
-          objKey = []
-          keyTrackDict[obj] = objKey
-        if objKey:
-          raise ("ERROR objKey for %s is non-empty %s, should not happen"
-           % (obj,objKey)
-          )
- 
-        # fill in keylist, first from parent:
-        objKey[:] = keyTrackDict[obj.parent][:]
-        # next add keys for the partricular object
-        #If it contains objects use the objects keyList, or make a new one
-        for tag,hicard in keyTuple:
-          if hicard == 1:
-            xxlist = [getattr(obj,tag)]
-          else:
-            xxlist = list(getattr(obj,tag))
-          for xx in xxlist:
-            if isinstance(xx,implClass):
-              ll =  keyTrackDict.get(xx)
-              if ll is None:
-                ll = []
-                keyTrackDict[xx] = ll
-              objKey.append(ll)
-            else:
-              objKey.append(xx)
-  
-      # add children to objs
-      for tag in dd['childlinks']:
-        objs.extend(getattr(obj,tag))
-      for tag in dd['singlekids']:
-        objs.append(getattr(obj,tag))
-  
-  
-  # make key:obj dictionary for second tree
-  inversDict = {}
-  top = objLists[1][0]
-  for obj,val in keyMaps[1].items():
-    if val or obj is top:
-      strKey = str(val)
-      if inversDict.has_key(strKey):
-        raise "key %s for obj %s is not unique" % (val,obj)
-      else:
-        inversDict[strKey] = obj
-    
-  for ii in (0,1):
-    top = objLists[ii][0]
-    keyTrackDict = keyMaps[ii]
-    
-    # put in extended keys for objects not in copied tree
-    # also add downlinks to keys  to ensure uniqueness
-    for obj,val in keyTrackDict.items():
-      if not val:
-        if obj is not top:
-          val[:] = obj.getExpandedKey()[:]
-      if isinstance(obj,ApplicationData):
-        val.append(obj.parent.keyPath[2])
-      val.append(obj.keyPath[2])
-      
-  # make comparison:
-  extras1 = []
-  diffs = {}
-  objMatches = {}
-  for obj in objLists[0]:
-    # set up
-    diffTags = []
-    objKey = keyMaps[0][obj]
-    clazz = obj.metaclass
-    dd = classData[clazz]
-    
-    # get equivalent object
-    strKey = str(objKey)
-    obj2 = inversDict.get(strKey)
-    if obj2 is None:
-      extras1.append(obj)
-      continue
-    else:
-      del inversDict[strKey]
-    
-    # check class
-    if obj.metaclass is not obj2.metaclass:
-      diffTags.append('__class__')
-      continue
-    
-    # check attributes
-    for tag in dd['attrs']:
-      if getattr(obj,tag) != getattr(obj2,tag):
-        diffTags.append(tag)
-        
-    for tag,hicard in dd['crosslinks']:
-      
-      # crosslinks - single valued
-      if hicard == 1:
-        vals = (getattr(obj,tag),getattr(obj2,tag))
-        if vals[0] is None:
-          if vals[1] is not None and not acceptSubset:
-            diffTags.append(tag)
-          
-        elif vals[1] is None:
-          diffTags.append(tag)
-        
-        else:
-          keys = [None,None]
-          for ii in (0,1):
-            vv = vals[ii]
-            kk = keyMaps[ii].get(vv)
-            if kk is None:
-              kk = vv.getExpandedKey
-            keys[ii] = kk
-          if keys[0] != keys[1]:
-            diffTags.append(tag)
-            
-      else:
-        # crosslinks multivalued
-        vals = (getattr(obj,tag),getattr(obj2,tag))
-        
-        if len(vals[0]) != len(vals[1]):
-          if not acceptSubset or len(vals[0]) > len(vals[1]):
-            diffTags.append(tag)
-            continue
-        
-        # local map dictionary
-        dd = {}
-        for vv in vals[1]:
-          kk = keyMaps[1].get(vv)
-          if kk is None:
-            kk = vv.getExpandedKey
-          dd[str(kk)] = None
-        
-        # check for non-matches
-        for vv in vals[0]:
-          kk = keyMaps[0].get(vv)
-          if kk is None:
-            kk = vv.getExpandedKey
-          if not dd.has_key(str(kk)):
-            diffTags.append(tag)
-            break
-    
-    # add to dictionaries
-    if diffTags:
-      diffs[obj] = diffTags
-    objMatches[obj] = obj2
-  
-  # return result
-  #extras2 = []
-  #for obj in inversDict.values():
-  #  if obj in objLists[1]:
-  #    extras2.append(obj)
-    
-  extras2 = inversDict.values()
-  if extras1 or extras2 or diffs:
-    return (extras1,extras2,diffs,objMatches)
-  else:
-    return False
-'''
-
  
 def newGuid(prefix = ''):
  
@@ -1171,7 +808,7 @@ def delayedLoadLinksCopy(objectDict, linkData, ignoreMissing=False):
             pass
           else:
             # link could not be handled
-            raise (
+            raise Exception(
              "%s: Out-of-subtree -to-one link %s cannot be copied"
              % (obj, name)
             )
@@ -1212,7 +849,7 @@ def delayedLoadLinksCopy(objectDict, linkData, ignoreMissing=False):
           
       if not done:
         # link could not be handled
-        raise (
+        raise Exception(
          "%s: Out-of-subtree link %s cannot be copied"
          % (obj, name)
         )
