@@ -10,6 +10,8 @@ from nijmegen.CASD import Constants as casdConstants
 from nijmegen.CASD import Util as casdUtil
 from memops.general import Io as genIo
 
+from ccp.lib import StructureIo
+
 from pdbe.deposition.dataFileImport.formatConverterWrapper import FormatConverterWrapper
 
 #casdNmrDir = casdConstants.casdNmrDir
@@ -316,12 +318,8 @@ def makeCcpnProject(entryName):
         
         if fcw is None:
           # dataIo version
-          if len(pdbPaths) == 1:
-            ensemble = casdUtil.loadPdbFile(ccpnProject.findFirstMolSystem(), 
-                                            pdbPaths[0])
-          else:
-            ensemble = casdUtil.loadPdbFiles(ccpnProject.findFirstMolSystem(), 
-                                             pdbPaths)
+          ensemble = StructureIo.getStructureFromFiles(
+                                 ccpnProject.findFirstMolSystem(), pdbPaths)
                                              
           if ensemble is None:
             print '### Skipping %s, no structures loaded' % entryName
@@ -534,10 +532,7 @@ def makeCcpnProject(info):
                                       suppressGeneralDataDir=True)
  
       pdbPaths = [os.path.join(structureDir,x) for x in pdbFiles]
-      if len(pdbPaths) == 1:
-        casdUtil.loadPdbFile(ccpnProject.findFirstMolSystem(), pdbPaths[0])
-      else:
-        casdUtil.loadPdbFiles(ccpnProject.findFirstMolSystem(), pdbPaths)
+      StructureIo.getStructureFromFiles(ccpnProject.findFirstMolSystem(), pdbPaths)
       
       print '### num files, ensembles', len(pdbPaths), len(ccpnProject.structureEnsembles)
       x = ccpnProject.findFirstStructureEnsemble()
